/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <qkeycode.h> 
#include <qstatusbar.h> 
#include "dbsheet.h" 
#include "mdi.h" 
#include "timerdlg.h" 
#include <qmessagebox.h>
#include "sqlcache.h" 
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qapplication.h>
#include <iostream.h>
#include <qtimer.h>

#include "saveas.xpm" 
#include "refresh.xpm" 
#include "sort.xpm" 
#include "filter.xpm" 
#include "sql.xpm" 


DBSheet::DBSheet(QWidget * parent, const QString& name)
       : Sheet(parent, name),query(QString::null),queryName(QString::null),
         dd(0), timer(0), timerInterval(60),
	 menuAdditionId(0),menuAdditionSeparatorId(0),refreshToggleMenuItem(0)
{
	// set the action menu
	fillActionMenu();
}

DBSheet::DBSheet(const ObjParm& parm, QWidget * parent, const QString& name)
       : Sheet(parm, parent, name),query(QString::null),
         queryName(QString::null),dd(0),
	 timer(0), timerInterval(60),
	 menuAdditionId(0),menuAdditionSeparatorId(0),refreshToggleMenuItem(0),
	 corButton(0)
{
	// set the action menu
	fillActionMenu();
}

void DBSheet::SQLErrorHandler()
{
	if (timer)
		if (timer->isActive())
		{
			// stop the timer if there is one running
			StartOrUpdateTimer(0);
			QMessageBox::information( 0, "Timer stopped!", 
				"Refresh timer stopped due to SQL error.",QMessageBox::Ok | QMessageBox::Default,0,0);
		}
}

void DBSheet::fillActionMenu()
{
        // set the action menu
        QPixmap saveAsIcon ;
        saveAsIcon = QPixmap( saveas );
        getActionMenu()->insertItem(saveAsIcon, "Save as...",this,SLOT(SaveAs()));
        getActionMenu()->insertSeparator();

 	QPixmap refreshIcon ;
        refreshIcon = QPixmap( refresh );
	QIconSet refreshIconSet;
	//refreshIconSet=QIconSet(refreshIcon); 	
        getActionMenu()->insertItem(refreshIcon, "Refresh",this,SLOT(retrieve()),CTRL+Key_R);

 	//QPixmap clearIcon ;
        //clearIcon = QPixmap( refresh );
	//QIconSet clearIconSet;
        refreshToggleMenuItem=getActionMenu()->insertItem("Log mode",this,SLOT(toggleLogMode()));
	getActionMenu()->setItemChecked(refreshToggleMenuItem,false);

        getActionMenu()->insertItem( "Refresh timer...",this,SLOT(TimerDialog()));

        QPixmap sortIcon ;
        sortIcon = QPixmap( sort );
	//QIconSet sortIconSet;
        getActionMenu()->insertItem(sortIcon, "Sort...",this,SLOT(SortDialog()));

        QPixmap filterIcon ;
        filterIcon = QPixmap( filter );
	//QIconSet filterIconSet;
        getActionMenu()->insertItem(filterIcon, "Filter...",this,SLOT(FilterDialog()));

        QPixmap sqlIcon ;
        sqlIcon = QPixmap( sql );
	//QIconSet sqlIconSet;
        getActionMenu()->insertItem(sqlIcon, "Show SQL...",this,SLOT(SQLDialog()));

	setupActionToolBar();
}

QPopupMenu * DBSheet::addGenerateSQLToActionMenu()
// This can be called when a descendant needs it
// It then only must add its own menus to the Generate SQL menu
{
        QPopupMenu * action_menu = getActionMenu();
        action_menu->insertSeparator();

        QPopupMenu * generate_menu = new QPopupMenu(this);
        action_menu->insertItem("Generate SQL",generate_menu);
	return generate_menu;
}

void DBSheet::adaptActionMenu()
{
	// This method is used by the dbtab to adapt the action menu
  	// according to the current tab
	QPopupMenu * action_menu = getActionMenu();
	// Take care of menu addition (e.g. Generate SQL)
	// Remove current menu addition if present before adding another 
	// one
	if (menuAdditionId!=0)
	{
		// remove menu item and its separator
		action_menu->removeItem(menuAdditionSeparatorId);
		menuAdditionSeparatorId=0;
		action_menu->removeItem(menuAdditionId);
		menuAdditionId=0;
	}
	if (dd->getMenuAddition())
	{
		menuAdditionSeparatorId=action_menu->insertSeparator();
		QPopupMenu* menu=dd->getMenuAddition();
		menuAdditionId=action_menu->insertItem(QString::fromLocal8Bit(menu->name()),menu);
	}

	// Take care of Log Mode setting (checked or not)
	if (dd->getLogMode()) getActionMenu()->setItemChecked(refreshToggleMenuItem,true);
	else getActionMenu()->setItemChecked(refreshToggleMenuItem,false);
}

void DBSheet::setupActionToolBar()
{
	QToolBar* tb = getActionToolBar();

        QPixmap saveAsIcon ;
        saveAsIcon = QPixmap( saveas );
        QToolButton * saveAs = new QToolButton( saveAsIcon, "Save as", 0,
                                            this, SLOT(SaveAs()),
                                            tb, "Save as" );
	saveAs->show();
	tb->addSeparator();

        QPixmap refreshIcon ;
        refreshIcon = QPixmap( refresh );
        QToolButton * refresh = new QToolButton( refreshIcon, "Refresh", 0,
                                            this, SLOT(retrieve()),
                                            tb, "Refresh" );
        refresh->show();

        QPixmap corIcon ;
        //corIcon = QPixmap( saveas );
        //corButton = new QToolButton( corIcon, 
		//			"Log mode", 0,
               //                         this, 
		//		        SLOT(toggleLogMode()),
                 //                       tb, "Refresh" );
	//corButton->setToggleButton(true);
	//corButton->setOn(true);
        //corButton->show();

        QPixmap sortIcon ;
        sortIcon = QPixmap( sort );
        QToolButton * sort = new QToolButton( sortIcon, "Sort", 0,
                                            this, SLOT(SortDialog()),
                                            tb, "Sort" );
        sort->show();

        QPixmap filterIcon ;
        filterIcon = QPixmap( filter );
        QToolButton * filter = new QToolButton( filterIcon, "Filter", 0,
                                            this, SLOT(FilterDialog()),
                                            tb, "Filter" );
        filter->show();

        QPixmap sqlIcon ;
        sqlIcon = QPixmap( sql );
        QToolButton * sql = new QToolButton( sqlIcon, "SQL", 0,
                                            this, SLOT(SQLDialog()),
                                            tb, "SQL" );
        sql->show();                                                                      
}

DBSheet::~DBSheet()
{
#ifdef DESTRACE
	cerr << "dbsheet destructor" << endl;
#endif
/*
        delete [] queryName;
	queryName=0;
        delete [] query;
	query=0;
*/
}

void DBSheet::setQueryName(const QString& name)
//void DBSheet::setQueryName(const char* name)
{
/*        if (queryName) 
	{
		delete [] queryName;
		queryName=0;
	}
        queryName=new char [strlen(name)+1];
        strcpy(queryName,name);
*/
	queryName=name;

	// set the query; get it from the SQLCache
	QString sqlstring;
	if (gSQLCache) sqlstring = gSQLCache->getSQL(name);
	else
	{
		QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
		return;
	}

	if (sqlstring!="")
		setQuery(sqlstring);
	else
	{
		// Unwind the waitCursors that may have been set elsewhere
                while (qApp->overrideCursor()) qApp->restoreOverrideCursor();
		QMessageBox::critical( 0, "SQL cache error", QString("The following SQL could not be found: ") + name,QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

void DBSheet::setQuery(const QString& q)
{
/*
        if (query) 
	{
		delete [] query;
		query=0;
	}
        query=new char [strlen(q)+1];
        strcpy(query,q);
*/
	query=q;
}

const QString& DBSheet::getQuery() const 
//const char * DBSheet::getQuery() const 
{
	return(query);
}

const QString& DBSheet::getQueryName() const 
//const char * DBSheet::getQueryName() const 
{
	return(queryName);
}

void DBSheet::retrieve()
{
	dd->setNoInitQuery(gMDI->getNoInitQuery());
	qApp->setOverrideCursor(waitCursor);
	statusBar()->message("Retrieving...");
	long rows=0;
	if (gConnObj) rows=dd->retrieve(getWinParm());
	else QMessageBox::critical( 0, "Database error", "You are not connected to a database!",QMessageBox::Ok | QMessageBox::Default,0,0);
	if (rows==-1) rows=0; // this is in case there was an error 
    	QString msg;
    	msg.setNum(rows);

	if (rows==1)
    		statusBar()->message(msg+" row retrieved.");
	else
    		statusBar()->message(msg+" rows retrieved.");

	qApp->restoreOverrideCursor();
}

void DBSheet::SortDialog()
{
	if (dd->SortDialog()) retrieve();
}

void DBSheet::FilterDialog()
{
        if (dd->FilterDialog()) 
	{
		adaptSheetTitle();
		retrieve();
	}
}

void DBSheet::StartOrUpdateTimer(int interval)
{
	if (interval==0)
	{
		timerInterval=60;
		delete timer;
		timer=0;
	}
	else
	{
		timerInterval=interval;
		if (!timer) 
		{
			timer=new QTimer(this);
			connect( timer, SIGNAL(timeout()),this, SLOT(retrieve()));
		}
		if (timer->isActive()) timer->changeInterval(timerInterval*1000);
		else timer->start(timerInterval*1000);
	}
	adaptSheetTitle();
}
	

void DBSheet::TimerDialog()
{
	TimerDlg * td = new TimerDlg(timerInterval,this);
	if (td->exec())
        {
		StartOrUpdateTimer(td->getInterval());
                delete td;
                td=0;
        }
        else
        {
                delete td;
                td=0;
        }
}

void DBSheet::adaptSheetTitle()
{
        QString tempTitle=getOriginalSheetTitle();
        // The && is necessary because one implementation of getFilter returns
        // "" and another return null in case when there is no filter
        if (dd->getFilter()!="" && dd->getFilter()!=QString::null)
                tempTitle=tempTitle+" [filtered]";

	if (timer)
		if (timer->isActive()) 
			tempTitle=tempTitle+" [timer]";

	if (dd->getLogMode())
		tempTitle=tempTitle+" [log]";

        setCaption(tempTitle);
}

void DBSheet::SQLDialog()
{
        dd->SQLDialog(getQueryName());
}

void DBSheet::removeDD()
{
        delete dd;
	dd=0;
}

DataDisplay* DBSheet::getDD() const
{
	return dd;
}

void DBSheet::assignDD(DataDisplay* pdd)
{
	dd=pdd;
}

void DBSheet::Print()
{
	dd->print();
}

void DBSheet::SaveAs()
{
	dd->save();
}

const bool DBSheet::getLogMode() const
{
        return(dd->getLogMode());
}

void DBSheet::toggleLogMode()
{
	if (getLogMode())
	{
//		corButton->setOn(false);
        	dd->setLogMode(false);
		getActionMenu()->setItemChecked(refreshToggleMenuItem,false);
	}
	else
	{
//		corButton->setOn(true);
		dd->setLogMode(true);
		getActionMenu()->setItemChecked(refreshToggleMenuItem,true);
	}
	adaptSheetTitle();
}
