/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "editor.h" 
#include <qtextstream.h> 
// For qt3
#include <qregexp.h> 
#include <qfile.h> 
#include <qmessagebox.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qpaintdevicemetrics.h>
#include <qpainter.h>
#include <qtextstream.h>
#include <qprinter.h>
#include <qfileinfo.h>

#include <stdlib.h>
#include <iostream.h>

#include "config.h"
#include "logondlg.h"

#include "saveas.xpm"
#include "execute.xpm"
#include "open.xpm"
#include "multilineedit.h" 


Editor::Editor(QWidget * parent, const QString& name)
//Editor::Editor(QWidget * parent, const char * name)
	: Sheet(parent, name), FileName(QString::null)
{
	construct();
	setCaption("New script");
}
Editor::Editor(QString fn, QWidget * parent, const QString& name)
//Editor::Editor(QString fn, QWidget * parent, const char * name)
	: Sheet(parent, name), FileName(QString::null)
{
	construct();
	loadFile(fn);
}

void Editor::construct()
{
	me = new MultiLineEdit(this);
	me->setWordWrap(QMultiLineEdit::WidgetWidth);
	me->setFocus();

	printer=new QPrinter;

	setCentralWidget(me);
	fillActionMenu();
}

Editor::~Editor()
{
	delete printer;
}

void Editor::loadFile(QString fn)
{
	FileName=fn;
	setCaption(FileName);
	QFile f (FileName);
	if (f.open(IO_ReadOnly))
	{
		QTextStream t(&f);
		QString s;
		while (!t.eof())
		{
			s+=t.readLine()+'\n';
		}
		me->setText(s);
		f.close();

		me->setEdited( FALSE );
	}
}

void Editor::closeEvent( QCloseEvent* ce )
{
	if ( !me->edited() ) 
	{
        	ce->accept();
        	return;
    	}

	setFocus();

	// Unwind the waitCursors that may have been set elsewhere
	while (qApp->overrideCursor()) qApp->restoreOverrideCursor();
	switch(QMessageBox::information( this, "SQL Editor message",
               "Do you want to save?",
               "Yes", "No", "Cancel",
               0, 1 ))
	{
    		case 0:
			save();
        		ce->accept();
        		break;
    		case 2:
    		default: // just for sanity
        		ce->ignore();
        		break;
    		case 1:
        		ce->accept();
        		break;
    	}
}

void Editor::fillActionMenu()
{
        // set the action menu
	getActionMenu()->insertSeparator();
        getActionMenu()->insertItem("New",this,SLOT(newFile()),CTRL+Key_N);
        QPixmap openIcon ;
        openIcon = QPixmap( openFile );
        getActionMenu()->insertItem(openIcon, "Open...",this,SLOT(open()),CTRL+Key_O);
        getActionMenu()->insertItem("Save",this,SLOT(save()),CTRL+Key_W);
        QPixmap saveIcon ;
        saveIcon = QPixmap( saveas );
        getActionMenu()->insertItem(saveIcon,"Save as...",this,SLOT(saveAs()));
	getActionMenu()->insertSeparator();
        getActionMenu()->insertItem("Find...",this,SLOT(Find()),CTRL+Key_F);
        getActionMenu()->insertItem("Replace...",this,SLOT(Replace()),CTRL+Key_R);
	getActionMenu()->insertSeparator();
        QPixmap executeIcon;
        executeIcon = QPixmap( exec );
        getActionMenu()->insertItem(executeIcon,"Execute...",this,SLOT(executeWithCurrentCredentials()),CTRL+Key_R);
        getActionMenu()->insertItem("Execute with other credentials...",this,SLOT(executeWithOtherCredentials()));

	setupActionToolBar();
}

void Editor::setupActionToolBar()
{
        QToolBar* tb = getActionToolBar();

        tb->addSeparator();

        QPixmap openIcon ;
        openIcon = QPixmap( openFile );
        QToolButton * open = new QToolButton( openIcon, "Open", 0,
                                            this, SLOT(open()),
                                            tb, "Open file" );

        open->show();

        QPixmap saveIcon ;
        saveIcon = QPixmap( saveas );
        QToolButton * save = new QToolButton( saveIcon, "Save", 0,
                                            this, SLOT(save()),
                                            tb, "Save" );
        save->show();

        tb->addSeparator();

        QPixmap executeIcon;
        executeIcon = QPixmap( exec );
        QToolButton * execute = new QToolButton( executeIcon, "Execute", 0,
                                            this, SLOT(executeWithCurrentCredentials()),
                                            tb, "Execute with current credentials" );
        execute->show();
}

void Editor::newFile()
{
        if ( me->edited() )
        {
        	switch(QMessageBox::information( this, "SQL Editor message",
               		"Do you want to save?",
               		"Yes", "No", "Cancel",
               		0, 1 ))
        	{
                	case 0:
                        	save();
                        	break;
                	case 1:
                	default: // just for sanity
                        	break;
                	case 2:
                        	return;
        	}
	}
	FileName=QString::null;
	setCaption("New script");
	me->clear();
	me->setEdited(false);
	me->setFocus();
}


void Editor::open()
{
	gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");
	QString fn;

	bool FileOK;
	do 
	{
		FileOK=true;
		fn = QFileDialog::getOpenFileName( path, "SQL scripts (*.sql)", this);
		if (!fn.isEmpty())
		{
			if (!QFileInfo(fn).exists()) 
			{
				FileOK=false;
				QMessageBox::critical (this, "File open error", 
							"'"+fn+"' does not exist!", 
							QMessageBox::Ok | QMessageBox::Default, 0, 0);
			}
			else if (!QFileInfo(fn).isReadable()) 
			{
				FileOK=false;
				QMessageBox::critical (this, "File open error", 
							"'"+fn+"' permission denied!", 
							QMessageBox::Ok | QMessageBox::Default, 0, 0);
			}
			else FileOK=true;
		}
	} while (!FileOK);
	if ( !fn.isEmpty() )
		loadFile( fn );
    	else
        	statusBar()->message( "Loading aborted", 2000 );
}
	
void Editor::saveAs()
{
	gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");

	QString fn;
	bool FileOK;
        do
        {
		FileOK=true;
		fn = QFileDialog::getSaveFileName( path, "SQL scripts (*.sql)", this );
                if (!fn.isEmpty())
                {
			// If the filename does not have an extension (i.e. contains a dot)
			// then add the extension ".sql"	
			if (QFileInfo(fn).fileName().find(".")==-1) fn=fn+".sql";
                        if (QFileInfo(fn).exists())
				if (!QFileInfo(fn).isWritable())
				{
					FileOK=false;
					QMessageBox::critical( 0, "File save error", 
							QString("The file ")+fn+
							QString(" already exists and you don't have any"
							" rights to overwrite it!\n\nPlease change the"
							" name of the file."),
							QMessageBox::Ok | QMessageBox::Default,0,0);
				}
				else if (QMessageBox::warning( 0, "File save warning", 
						QString("The file ")+fn+
						QString(" already exists!\n\nDo you want to overwrite?"),
						QMessageBox::Yes,QMessageBox::No | QMessageBox::Default,0)==QMessageBox::Yes)
					FileOK=true;
				else FileOK=false;
                        else FileOK=true;
                }
        } while (!FileOK);

	if ( !fn.isEmpty() ) 
	{
		FileName = fn;
        	save();
    	} 
	else 
	{
        	statusBar()->message( "Saving aborted", 2000 );
    	}
}

void Editor::save()
{
	if ( FileName.isEmpty() ) 
	{
		saveAs();
		return;
	}

	QString text = me->text();
	QFile f( FileName );
	if ( !f.open( IO_WriteOnly ) ) 
	{
		statusBar()->message( QString("Could not write to %1").arg(FileName),2000 );
		return;
	}

	QTextStream t( &f );
	t << text;
	f.close();

	me->setEdited( FALSE );

	setCaption( FileName );

	statusBar()->message( QString( "File %1 saved" ).arg( FileName ), 2000 );
}

void Editor::Print()
{
    const int Margin = 10;
    int pageNo = 1;

    if ( printer->setup(this) ) {               // printer dialog
        statusBar()->message( "Printing..." );
        QPainter p;
        if( !p.begin( printer ) )
            return;                             // paint on printer

        p.setFont( me->font() );
        int yPos        = 0;                    // y position for each line
        QFontMetrics fm = p.fontMetrics();
        QPaintDeviceMetrics metrics( printer ); // need width/height
                                                // of printer surface
        for( int i = 0 ; i < me->numLines() ; i++ ) {
            if ( Margin + yPos > metrics.height() - Margin ) {
                QString msg( "Printing (page " );
                msg += QString::number( ++pageNo );
                msg += ")...";
                statusBar()->message( msg );
                printer->newPage();             // no more room on this page
                yPos = 0;                       // back to top of page
            }
            p.drawText( Margin, Margin + yPos,
                        metrics.width(), fm.lineSpacing(),
                        ExpandTabs | DontClip,
                        me->textLine( i ) );
            yPos = yPos + fm.lineSpacing();
        }
        p.end();                                // send job to printer
        statusBar()->message( "Printing completed", 2000 );
    } else {
        statusBar()->message( "Printing aborted", 2000 );
    }
}

void Editor::execute(const QString &un, const QString &pw, const QString &tns)
{
        if ( me->edited() )
        {
                switch(QMessageBox::information( this, "SQL Editor message",
			"You must save this script before executing it.\n"
                        "Do you want to save?",
                        "Yes", "Cancel",
                        0, 1 ))
                {
                        case 0:
                                save();
                                break;
                        case 1:
                        default: // just for sanity
                                return;
                }
        }

        gConfigFile->setGroup("MISC");
        QString sqlplus=gConfigFile->readEntry("sqlplus","");
        QString connectString=un+"/"+pw;
        if (!tns.isEmpty()) connectString=connectString+"@"+tns;

        sqlplus.replace(QRegExp("<SPC>")," ");
        sqlplus.replace(QRegExp("<USERNAME>"),un);
        sqlplus.replace(QRegExp("<TNSALIAS>"),tns);
        sqlplus.replace(QRegExp("<CONNECTSTRING>"),connectString + " @" + FileName);
        system(sqlplus);
}

void Editor::executeWithCurrentCredentials()
{
	execute(gUsername, gPassword, gTNSAlias);
}

void Editor::executeWithOtherCredentials()
{
        gConfigFile->setGroup("CONNECTION");
        QString TNSAliasList=gConfigFile->readEntry("connect_strings","");
	LogonDlg *ld = new LogonDlg(this,"logondlg",gUsername,gPassword,gTNSAlias,TNSAliasList);	
	if (ld->exec()) execute(ld->getUsername(), ld->getPassword(), ld->getTNSAlias());
}

void Editor::Find()
{
	me->findDialog();
}

void Editor::Replace()
{
	me->replaceDialog();
}
