/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "formview.h" 
#include "general.h" 
#include <qlabel.h> 
#include <qpoint.h> 
#include <qstring.h> 
#include <iostream.h> 
#include <qregexp.h> 
#include <qmultilineedit.h> 
#include <qevent.h> 
#include <qobjcoll.h> 
#include <qlist.h> 
#include <qpaintdevicemetrics.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>
// For qt3
#include <qpainter.h>

//FormView::FormView(int cols,int lw, int tw, QWidget * parent, const char * name,WFlags f)
FormView::FormView(int cols,int lw, int tw, QWidget * parent, 
		   const QString& name,WFlags f)
	:QScrollView(parent,name.local8Bit(),f),columns(cols),
	rows(0),NoOfRecords(0),currCol(1),
	currRow(1),fieldDistX(5),fieldDistY(10),maxLabelWidth(lw),
	maxTextWidth(tw),
	LabelTextDist(5),maxFieldHeight(14),recDist(20)
{
	resizeContents(width(),height());
	RightMostPixel=width();
	LowestPixel=height();
	maxFieldWidth=maxLabelWidth+LabelTextDist+maxTextWidth;
	enableClipper(TRUE);
	setVScrollBarMode(QScrollView::Auto);
	setHScrollBarMode(QScrollView::Auto);
	start.setX(10);
	start.setY(10);
	currLoc=start;
	orient=horizontal;
      	viewport()->setBackgroundMode(PaletteBase);
}

FormView::~FormView()
{
#ifdef DESTRACE
	cerr << "FormView destructor" << endl;
#endif
}

void FormView::setText(const QString label, const QString text, bool single, int w, int h, bool wrap)
{
	QLabel *l;
	QLabel *t;
	if (single)
	{
		if (currCol>1)
			currLoc=QPoint(start.x(),currLoc.y()+maxFieldHeight+fieldDistY);
		currCol=columns;
                l = new QLabel(label,viewport());
                l->resize(maxLabelWidth,maxFieldHeight);
		l->setBackgroundMode(PaletteBase);
                addChild(l,currLoc.x(),currLoc.y());

                l->setAlignment(QFrame::AlignLeft|QFrame::AlignVCenter);
                t = new QLabel(text,viewport());

		t->resize(w,h);
		// Change alignment according to data (number or alpha)
		//QRegExp r = QRegExp("^[0-9][0-9]*$",false,false);
		//if (text.contains(r) > 0) t->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
		if (General::isNumber(text)) t->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
		t->setFrameStyle(QFrame::Box|QFrame::Plain);
		t->setBackgroundMode(PaletteBase);
		addChild(t,currLoc.x()+maxLabelWidth+LabelTextDist,currLoc.y());
	}
	else
	{
	        l = new QLabel(label,viewport());
       		l->resize(maxLabelWidth,maxFieldHeight);
		l->setBackgroundMode(PaletteBase);
      		addChild(l,currLoc.x(),currLoc.y());

  		l->setAlignment(QFrame::AlignLeft|QFrame::AlignVCenter);
        	t = new QLabel(text,viewport());

		t->resize(maxTextWidth,maxFieldHeight);
		// Change alignment according to data (number or alpha)
		//QRegExp r = QRegExp("^[0-9][0-9]*$",false,false);
		//if (text.contains(r) > 0) t->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
		if (General::isNumber(text)) t->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
		if (wrap) 
			{
			t->setAlignment(QFrame::AlignTop|QFrame::AlignLeft|QFrame::WordBreak);
			}
		t->setFrameStyle(QFrame::Box|QFrame::Plain);
		t->setBackgroundMode(PaletteBase);
		addChild(t,currLoc.x()+maxLabelWidth+LabelTextDist,currLoc.y());
	}
	
	if (orient==horizontal)
	{
		if (currCol<columns)
		{
			currLoc.setX(currLoc.x()+maxFieldWidth+fieldDistX);
			currCol++;
			RightMostPixel=currLoc.x();
		}
		else
		// next line
		{
			if (single)
				RightMostPixel=currLoc.x()+maxLabelWidth+LabelTextDist+w+fieldDistX;
			else
				RightMostPixel=currLoc.x()+maxFieldWidth+fieldDistX;
			if (single)
				currLoc=QPoint(start.x(),currLoc.y()+h+fieldDistY);
			else
				currLoc=QPoint(start.x(),currLoc.y()+maxFieldHeight+fieldDistY);
			currCol=1;
			if (single)
				LowestPixel=currLoc.y()+h+fieldDistY;
			else
				LowestPixel=currLoc.y()+maxFieldHeight+fieldDistY;
		}
	}
	if (RightMostPixel>contentsWidth())
		resizeContents(RightMostPixel,contentsHeight());
	if (LowestPixel>contentsHeight())
		resizeContents(contentsWidth(),LowestPixel);

}

void FormView::setMaxLabelWidth(int lw)
{
	maxLabelWidth=lw;
	maxFieldWidth=maxLabelWidth+LabelTextDist+maxTextWidth;
	
}

void FormView::setMaxTextWidth(int tw)
{
	maxTextWidth=tw;
	maxFieldWidth=maxLabelWidth+LabelTextDist+maxTextWidth;
}

void FormView::setLabelTextDist(int d)
{
	LabelTextDist=d;
	maxFieldWidth=maxLabelWidth+LabelTextDist+maxTextWidth;
}

void FormView::setMaxFieldHeight(int h)
{
	maxFieldHeight=h;
}

void FormView::setField(const QString label, const QString text)
{
        QLabel *t;
	if (maxLabelWidth!=0)
	{
        	QLabel *l;
        	l = new QLabel(label,viewport());
        	l->resize(maxLabelWidth,maxFieldHeight);
		l->setBackgroundMode(PaletteBase);
		l->show();
        	addChild(l,currLoc.x(),currLoc.y());

        	l->setAlignment(QFrame::AlignLeft|QFrame::AlignVCenter);
	}

        t = new QLabel(text,viewport());

        t->resize(maxTextWidth,maxFieldHeight);
	t->setBackgroundMode(PaletteBase);
	t->show();
        // Change alignment according to data (number or alpha)
        //QRegExp r = QRegExp("^[0-9,.][0-9,.]*$",false,false);
        //QRegExp r = QRegExp("^[0-9][0-9]*.?[0-9]*$",false,false);
        //if (text.contains(r) > 0) t->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        if (General::isNumber(text)) t->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        t->setFrameStyle(QFrame::Box|QFrame::Plain);
        addChild(t,currLoc.x()+maxLabelWidth+LabelTextDist,currLoc.y());

        if (orient==horizontal)
        {
                if (currCol<columns)
                {
                        currLoc.setX(currLoc.x()+maxFieldWidth+fieldDistX);
                        currCol++;
                        RightMostPixel=currLoc.x();
                }
                else
                // next line
                {
                        RightMostPixel=currLoc.x()+maxFieldWidth+fieldDistX;
                        currLoc=QPoint(start.x(),currLoc.y()+maxFieldHeight+fieldDistY);
                        currCol=1;
                        LowestPixel=currLoc.y()+maxFieldHeight+fieldDistY;
                }
        }
        if (RightMostPixel>contentsWidth())
                resizeContents(RightMostPixel,contentsHeight());
        if (LowestPixel>contentsHeight())
                resizeContents(contentsWidth(),LowestPixel);
}

void FormView::setSingleField(const QString label, const QString text, int w, int h, bool wrap)
{
    QMultiLineEdit *t;
    if (currCol>1)
    	currLoc=QPoint(start.x(),currLoc.y()+maxFieldHeight+fieldDistY);
    currCol=columns;
	if (maxLabelWidth!=0)
	{
        QLabel *l;
        l = new QLabel(label,viewport());
        l->resize(maxLabelWidth,maxFieldHeight);
	l->setBackgroundMode(PaletteBase);
	l->show();
        addChild(l,currLoc.x(),currLoc.y());

        l->setAlignment(QFrame::AlignLeft|QFrame::AlignVCenter);
	}
    t = new QMultiLineEdit(viewport());
	t->setText(text);

    t->resize(w,h);
	t->show();
    // Change alignment according to data (number or alpha)
    //QRegExp r = QRegExp("^[0-9,.][0-9,.]*$",false,false);
    //QRegExp r = QRegExp("^[0-9][0-9]*.?[0-9]*$",false,false);
    //if (text.contains(r) > 0) t->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
    if (General::isNumber(text)) t->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
    if (wrap)
    {
    	t->setAlignment(QFrame::AlignTop|QFrame::AlignLeft|QFrame::WordBreak);
		t->setWordWrap(QMultiLineEdit::WidgetWidth);
    }

    t->setFrameStyle(QFrame::Box|QFrame::Plain);
    addChild(t,currLoc.x()+maxLabelWidth+LabelTextDist,currLoc.y());

    if (orient==horizontal)
    {
    	if (currCol<columns)
        {
        	currLoc.setX(currLoc.x()+maxFieldWidth+fieldDistX);
            currCol++;
            RightMostPixel=currLoc.x();
		}
        else
        // next line
        {
        	RightMostPixel=currLoc.x()+maxLabelWidth+LabelTextDist+w+fieldDistX;
            currLoc=QPoint(start.x(),currLoc.y()+h+fieldDistY);
            currCol=1;
            LowestPixel=currLoc.y()+h+fieldDistY;
        }
	}
    if (RightMostPixel>contentsWidth())
    	resizeContents(RightMostPixel,contentsHeight());
    if (LowestPixel>contentsHeight())
    	resizeContents(contentsWidth(),LowestPixel);
}

void FormView::newRecord()
{
	if (currCol==1)
		currLoc=QPoint(start.x(),currLoc.y()-fieldDistY+recDist);
	else
		currLoc=QPoint(start.x(),currLoc.y()+recDist);
	currCol=1;
}

void FormView::setRecDist(int rd)
{
	recDist=rd;
}

void FormView::contentsMousePressEvent ( QMouseEvent * e)
{
	if (e->button()==RightButton)
		emit rightButtonPressed(e->globalPos());
}

void FormView::clear()
{
    	QObjectList *list = viewport()->queryList("QLabel");
    	QObjectListIt it( *list );          // iterate over the labels
    	QObject * obj;
    	while ( (obj=it.current()) != 0 ) { // for each found object...
        	++it;
        	delete obj;
    	}
    	delete list;                        // delete the list, not the objects
	list=0;

    	//resizeContents(width(),height());
    	resizeContents(100,30);
    	RightMostPixel=width();
    	LowestPixel=height();
    	currLoc=start;
	currCol=1;
	currRow=1;
	enableClipper(TRUE);
}

void FormView::printData(const QString& OutputFilename, const QString& docName, const QPrinter::Orientation& orient)
{
        const int Margin = 10;
        QPrinter *printer = new QPrinter();
        printer->setOutputFileName(OutputFilename);
        printer->setDocName(docName);
        printer->setOrientation(orient);

        if ( printer->setup(this) )
        {
                QPainter p;
                p.begin( printer );                     // paint on printer
                //int yPos        = 0;                    // y position for each line
                QFontMetrics fm = p.fontMetrics();
                QPaintDeviceMetrics metrics( printer ); // need width/height
                                                        // of printer surface

        	QObjectList *list = viewport()->queryList("QLabel");
        	QObjectListIt it( *list );          // iterate over the labels
        	QObject * obj;
		QLabel* lbl;
        	while ( (obj=it.current()) != 0 ) 
		{ // for each found object...
                	++it;
			lbl = (QLabel*)obj;
			if (lbl->frameStyle()!=QFrame::NoFrame) // so it's a text
			{
				p.drawRect( Margin+childX(lbl), Margin + childY(lbl),lbl->width(),
					    fm.lineSpacing()+fm.underlinePos());
				int align;
                        	QString text=lbl->text();
                        	//QRegExp r = QRegExp("^[0-9][0-9]*$",false,false);
                        	//if (text.contains(r) > 0)
                        	if (General::isNumber(text))
                        		align=ExpandTabs|AlignRight;
                        	else
                        		align=ExpandTabs;
				p.drawText( Margin+childX(lbl), Margin + childY(lbl),lbl->width(),
                               		    lbl->height(), align , text);
			}
			else // it's a label
			{
				p.drawText( Margin+childX(lbl), Margin + childY(lbl),lbl->width(),
                        		    lbl->height(), ExpandTabs , lbl->text());
			}
		}

		// print all multilineedits
                QObjectList *list_mle = viewport()->queryList("QMultiLineEdit");
                QObjectListIt it_mle( *list_mle );          // iterate over the labels
                QMultiLineEdit* mle;
                while ( (obj=it_mle.current()) != 0 )
                { // for each found object...
                        ++it_mle;
                        mle = (QMultiLineEdit*)obj;
                        p.drawRect( Margin+childX(mle), Margin + childY(mle),mle->width(),
                                    mle->height());
                        p.drawText( Margin+childX(mle), Margin + childY(mle),mle->width(),
                                    mle->height(), ExpandTabs , mle->text());

        	}
        	delete list;                        // delete the list, not the objects
		list=0;
        	delete list_mle;                        // delete the list, not the objects
		list_mle=0;
		p.end();                                // send job to printer
	}
}

void FormView::saveData()
{
        QString filename=QFileDialog::getSaveFileName("", "*.txt", this );
        if ( !filename.isNull() )
        {
                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream ts(&f);

                	QObjectList *list = viewport()->queryList("QFrame");
                	QObjectListIt it_label( *list );          // iterate over the labels
                	QObjectListIt it_text( *list );          // iterate over the text fields
                	QObject * obj;
                	QLabel* lbl;
                	QMultiLineEdit* mle;
			
			// save the labels
                        while ( (obj=it_label.current()) != 0 )
                        { // for each found object...
                                ++it_label;
                                lbl = (QLabel*)obj;
                                if (lbl->frameStyle()==QFrame::NoFrame) // it's a label
                                {
                                        ts << lbl->text() << "\t";
                                }
                        }
			ts << endl << endl;

			// save the text
                	while ( (obj=it_text.current()) != 0 )
                	{ // for each found object...
                        	++it_text;
				if ((obj->className())==QString("QLabel"))
				{
                        		lbl = (QLabel*)obj;
                        		if (lbl->frameStyle()!=QFrame::NoFrame) // so it's a text
                        		{
						if (lbl->text()==QString::null)
							ts << "(null)" << "\t";
						else
							ts << lbl->text()  << "\t";
					}
				}
				else if (obj->className()==QString("QMultiLineEdit"))
				{
					mle = (QMultiLineEdit *)obj;
					if (lbl->text()==QString::null)
						ts << "(null)" << "\t";
					else
						ts << mle->text() << "\t";
				}
                        }

                	delete list;                        // delete the list, not the objects
			list=0;
		}
        }
}

