/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "mdi.h"
#include "w_db_overview.h"
#include <stdlib.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qobjcoll.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qmultilinedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <iostream.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qevent.h>
#include "sqlcache.h"
#include "qt_oraconnect.h"
#include <qmenudata.h>
#include <qdom.h>
// For qt3
#include <qregexp.h>

#include "config.h"
#include "dblist.h"
#include "dbform.h"
#include "dbtab.h"                 
#include "w_dba_users.h"
#include "w_dba_roles.h"
#include "w_dba_tablespaces.h"
#include "w_dba_rollback_segs.h"
#include "w_dba_data_files.h"
#include "w_all_objects.h"
#include "w_v_log.h"                                                           
#include "w_v_session.h"
#include "w_locktree.h"
#include "usersqllist.h"
#include "generate_sql_cache_entry.h"
#include "w_invalid_objects.h"
#include "gensql.h"                                                           
#include "editor.h"                                                           
#include "scriptdlg.h" 
#include "w_free_space.h"

#include "sqlcache.h"
#include "logondlg.h"
#include "w_performance.h"
#include "w_top_sql.h"
#include "w_v_complete_list.h"

#include "conn.xpm"
#include "quit.xpm"
#include "users.xpm"
#include "tbs.xpm"
#include "rbs.xpm"
#include "dbf.xpm"
#include "sessions.xpm"
#include "userdefsql.xpm"
#include <qglobal.h>
#if QT_VERSION > 0x020302
#define QT3
#endif

mdi::mdi()
    : Window( 0, 0 ), SheetList(0), SheetSeqNo(0), ws(0), ToolBar(0), 
	action(0), ActionMenuID(0), actionToolBar(0), windows(0), FileMenuID(0),
	activeSheet(0),noInitQuery(false),NIQItem(0),tools(0),
	mdiBaseTitle(QString::null),vdollar(0),vdollarMenuCache(0)
{
    // Create mdi workspace
    QVBox* vb = new QVBox( this );
    vb->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
    ws = new QWorkspace( vb );
    setCentralWidget( vb );
    connect(ws,SIGNAL(windowActivated(QWidget*)),this,SLOT(SheetActivated(QWidget*)));                               

	setCaption("(NOT CONNECTED)");

	createToolBar();
	vdollarMenuCache = new QIntDict<QString>(1);
	createMenu();
	//menuBar()->setItemEnabled(1,false);
	changeMenuState(false);

	// This will contain the list of the sheets that have been opened
	SheetList = new QIntDict<Sheet>(53);	
	SheetSeqNo=0; //number attributed to the next sheet

    statusBar()->message( "Ready", 2000 );
	
}


mdi::~mdi()
{
	try
	{
#ifdef DESTRACE
		cerr << "mdi destructor" << endl;
#endif
		saveVDollarMenu();
    		// now delete all the sheets
	    	SheetList->setAutoDelete(true);
    		delete SheetList;
		SheetList=0;
		activeSheet=0;
       		delete gSQLCache;
       		gSQLCache=0;
       		delete gConnObj;
       		gConnObj=0;
       		gMDI->setCaption("(NOT CONNECTED)");
	}
	catch(oerr& e)
	{
		QMessageBox::critical( 0, "SQL error while exiting", QString("Oracle error: ") + ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
   	}
}


void mdi::commandLineLogon(const QString& connectString)
{
        QString cs(connectString);

        // extract the username from the connect string
        int slashpos=cs.find("/",0,false);
        if (slashpos==-1)
        {
                // no slash(no password) but there may be a @
                int atpos=cs.find("@",0,false);
                if (atpos==-1)
                {
                        // no @ either so the whole connect string is the username
                        gUsername=cs;
                        gPassword="";
                        gTNSAlias="";
                }
                else
                {
                        // there is a @ so there is a useranme and a tns alias
                        gUsername=cs.left(atpos);
                        gPassword="";
                        gTNSAlias=cs.mid(atpos+1,cs.length()-atpos);
                }
        }
        else
        {
                // There is a slash, so there is a username and password though they maybe empty (OPS logon)
                gUsername=cs.left(slashpos);
                int atpos=cs.find("@",0,false);
                if (atpos==-1)
                {
                        // no @ so the rest of the connect string is the password
                        gPassword=cs.mid(slashpos+1,cs.length()-slashpos+1);
                        gTNSAlias="";
                }
                else
                {
                        // There is also an @
                        gPassword=cs.mid(slashpos+1,atpos-slashpos-1);
                        gTNSAlias=cs.mid(atpos+1,cs.length()-atpos);
                }
        }
        // connect to the database
	bool Connected=false;

	Connected=connectToDB();
	if (Connected)
	{
                setupSQLCache();
		registerApplication();
		customizeSession();
                setupMDITitle();
		changeMenuState(true);
        }
	else
	{
		interactive_logon();
	}
}


bool mdi::event ( QEvent * e ) 
{
	// If we get a user event then we know it is 
	// to logon
	if (e->type()==QEvent::User)
	{
		if (gCommandLineArg=="")
		{
			interactive_logon();
		}
		else
		{
			commandLineLogon(gCommandLineArg);
		}
		//delete e; // this causes a seg fault under LINUX
		// so for the moment I take it out even though this creates a mem leak
		return(true);
	}
	else
		return Window::event(e);
}

uint mdi::registerSheet(const Sheet *s)
{
	SheetSeqNo++;
	SheetList->insert(SheetSeqNo,s);
	return SheetSeqNo;
}

bool mdi::unregisterSheet(const uint index)
{
	bool retval= SheetList->remove(index);
	// if there are no more sheets then set actiontoolbar to 0
	if (SheetList->isEmpty()) 
	{
		actionToolBar=0;
		action=0;
	}
	return retval;
}

// Original void mdi::deleteAllSheets()
bool mdi::deleteAllSheets()
{
        QIntDictIterator<Sheet> it( *SheetList ); // iterator for dict
        while ( it.current() ) 
	{
/* Change of meaning for WDestructive Close:
** In QT2 this meant the window would be destroyed right away but
** in QT3 the actual destruction is delayed to later and thus
** my loop will never end. Therfore I had to force the close using 
** the close(true) call.
*/
#ifdef QT3
                if (!it.current()->close(true))
#else
                if (!it.current()->close())
#endif
		{
			return false;
		}
		// Iteration moves on because the close() removes
		// sheets from list
	}
        	
    	delete SheetList;
	SheetList=0;
        SheetList = new QIntDict<Sheet>(53);
        SheetSeqNo=0; //number given to the next sheet

	return true;

/* Original
    	SheetList->setAutoDelete(true);
    	delete SheetList;
	SheetList=0;
        SheetList = new QIntDict<Sheet>(53);
        SheetSeqNo=0; //number given to the next sheet
*/
}

QWorkspace* mdi::getWorkspace()
{
	return ws;
}

void mdi::SheetActivated(QWidget * w)
{
	if (!w) return; // This was necessary for Qt 2.2.0 where w could be 0
	activeSheet=(Sheet*)w;
	((Sheet*)w)->replaceMDIActionMenu();
	((Sheet*)w)->replaceMDIActionToolbar();
}                                                                               

void mdi::createToolBar()
{
    ToolBar = new QToolBar(this,"Main toolbar");
    QPixmap connectIcon, quitIcon,  usersIcon, rbsIcon, dbfIcon;
    QPixmap sessionsIcon, userdefsqlIcon, tbsIcon;
 
    connectIcon = QPixmap( conn );
    QToolButton * conn = new QToolButton( connectIcon, "Connect", 0,
                                              this, SLOT(postInteractiveLogon()),
                                              ToolBar, "Connect" );
 
        quitIcon = QPixmap( quit );
        QToolButton * quit = new QToolButton( quitIcon, "Quit", 0,
                                              this, SLOT(quitDBAComp()),
                                              ToolBar, "Quit" );
 
    //ToolBar->addSeparator();
    ToolBar->addSeparator();
 
        usersIcon = QPixmap( users );
        QToolButton * users = new QToolButton( usersIcon, "Users", 0,
                                               this, SLOT(dba_users()),
                                               ToolBar, "Users" );
 
        tbsIcon = QPixmap( tbs );
        QToolButton * tbs = new QToolButton( tbsIcon, "Tablespaces", 0,
                                               this, SLOT(dba_tablespaces()),
                                               ToolBar, "Tablespaces" );
 
        rbsIcon = QPixmap( rbs );
        QToolButton * rbs = new QToolButton( rbsIcon, "Rollback segs", 0,
                                               this, SLOT(dba_rollback_segs()),
                                               ToolBar, "Rollback segs" );
 
        dbfIcon = QPixmap( dbf );
        QToolButton * dbf = new QToolButton( dbfIcon, "Datafiles", 0,
                                               this, SLOT(dba_data_files()),
                                               ToolBar, "Datafiles" );
 
 
    //ToolBar->addSeparator();
    ToolBar->addSeparator();                                                                                        
 
        sessionsIcon = QPixmap( sessions );
        QToolButton * sessions = new QToolButton( sessionsIcon, "Sessions", 0,
                                               this, SLOT(v_session()),
                                               ToolBar, "Sessions" );
 
    //ToolBar->addSeparator();
    ToolBar->addSeparator();
 
        userdefsqlIcon = QPixmap( userdefsql );
        QToolButton * userdefsql = new QToolButton( userdefsqlIcon, "User defined SQL",0,
                                               this, SLOT(UserDefinedSQL()),
                                               ToolBar, "User defined SQL" );
 
    ToolBar->show();
}                                                                                                                     
void mdi::createMenu()
{
    QPixmap connectIcon, quitIcon,  usersIcon, rbsIcon, dbfIcon;
    QPixmap sessionsIcon, userdefsqlIcon, tbsIcon;

    QPopupMenu * file = new QPopupMenu(this);
    ///////////////////////////////////////////////////////
    ///////     File
    //////////////////////////////////////////////////////
    FileMenuID=menuBar()->insertItem( "&File", file );
        file->insertItem( "New", this, SLOT(newDBAComp()),CTRL+Key_R);
        file->insertItem( "Disconnect", this, SLOT(logoff()),CTRL+Key_D);
	connectIcon=QPixmap( conn );
        file->insertItem(connectIcon, "Connect", this, SLOT(postInteractiveLogon()),CTRL+Key_N);
        //file->insertSeparator();
        //file->insertItem( "Print Setup...");
        file->insertSeparator();
	quitIcon = QPixmap( quit );
        file->insertItem(quitIcon, "Quit", this, SLOT(quitDBAComp()), CTRL+Key_Q );
 
    ///////////////////////////////////////////////////////
    ////////    Action
    ///////////////////////////////////////////////////////
    //action = new QPopupMenu(this);
    //menuBar()->insertItem( "&Action", action );
 
    ///////////////////////////////////////////////////////
    ////////    Database
    ///////////////////////////////////////////////////////
    QPopupMenu * database = new QPopupMenu(this);
    menuBar()->insertItem( "&Database", database );
        database->insertItem( "Overview", this, SLOT(database_overview()) );
        database->insertSeparator();
	usersIcon = QPixmap( users );
        database->insertItem(usersIcon, "Users", this, SLOT(dba_users()), CTRL+Key_U );
        database->insertItem( "Roles", this, SLOT(dba_roles()));
        database->insertItem( "Profiles", this, SLOT(dba_profiles()));
        database->insertSeparator();

	tbsIcon = QPixmap( tbs );
        database->insertItem(tbsIcon, "Tablespaces", this, SLOT(dba_tablespaces()));
	rbsIcon = QPixmap( rbs );
        database->insertItem(rbsIcon, "Rollback segs", this, SLOT(dba_rollback_segs()));
	dbfIcon = QPixmap( dbf );
        database->insertItem(dbfIcon, "Datafiles", this, SLOT(dba_data_files()));
        database->insertItem( "Redo logs", this, SLOT(v_log()));
        database->insertItem( "Control files", this, SLOT(v_controlfile()));
        database->insertSeparator();
	QPopupMenu * backup = new QPopupMenu(this);
        database->insertItem( "Backup",backup);
		backup->insertItem("v$backup&&v$datafile",this,SLOT(v_backup_v_datafile()));
		backup->insertItem("v$backup",this, SLOT(v_backup()));
		backup->insertItem("v$backup_corruption",this,SLOT(v_backup_corruption()));
		backup->insertItem("v$backup_datafile",this,SLOT(v_backup_datafile()));
		backup->insertItem("v$backup_device",this,SLOT(v_backup_device()));
		backup->insertItem("v$backup_piece",this,SLOT(v_backup_piece()));
		backup->insertItem("v$backup_redolog",this,SLOT(v_backup_redolog()));
		backup->insertItem("v$backup_set",this,SLOT(v_backup_set()));
	QPopupMenu* recovery = new QPopupMenu(this);
        database->insertItem( "Recovery",recovery);
		recovery->insertItem("v$recover_file",this,SLOT(v_recover_file()));
		recovery->insertItem("v$recovery_log",this,SLOT(v_recovery_log()));
		recovery->insertItem("Datafile recovery status",this,SLOT(datafile_recovery_status()));
		recovery->insertItem("v$recovery_file_status",this,SLOT(v_recovery_file_status()));
		recovery->insertItem("v$recovery_status",this,SLOT(v_recovery_status()));
		
        database->insertSeparator();
        database->insertItem( "All objects", this, SLOT(all_objects())); 
 
    QPopupMenu * instance = new QPopupMenu(this);
    menuBar()->insertItem( "&Instance", instance );
	instance->insertItem("&Overview",this,SLOT(instance_overview()));
	instance->insertSeparator();
	instance->insertItem("&SGA",this,SLOT(sga()));
	instance->insertSeparator();
	sessionsIcon = QPixmap( sessions );
        instance->insertItem(sessionsIcon, "Sessions", this, SLOT(v_session()),CTRL+Key_S );
	instance->insertItem("Sessions and &Processes",this,SLOT(sessions_processes()));
	instance->insertItem("Sessions and &Transactions",this,SLOT(sessions_transactions()));
	instance->insertSeparator();
	instance->insertItem("&Locks",this,SLOT(locks()));
	instance->insertItem("&Locktree",this,SLOT(locktree()));
	instance->insertSeparator();
    	QPopupMenu * audit = new QPopupMenu(this);
    	instance->insertItem("&Audit",audit);
		audit->insertItem("Audit options",this,SLOT(audit_options()));
		audit->insertItem("Object Audit options",this,SLOT(object_audit_options()));
		audit->insertItem("Default Audit options",this,SLOT(default_audit_options()));
		audit->insertSeparator();
		audit->insertItem("Audit trail",this,SLOT(audit_trail()));
		audit->insertSeparator();
		audit->insertItem("Audit sessions",this,SLOT(audit_sessions()));
		audit->insertItem("Audit objects",this,SLOT(audit_objects()));
	instance->insertSeparator();
	instance->insertItem("&MTS",this,SLOT(mts()));

    ///////////////////////////////////////////////////////
    ////////        performance
    ///////////////////////////////////////////////////////
    QPopupMenu * performance = new QPopupMenu(this);
    menuBar()->insertItem( "&Performance", performance );
        performance->insertItem("Indicators", this, SLOT(perf_indicators()));
        performance->insertItem("Hot files", this, SLOT(hot_files()));
        performance->insertItem("Top SQL", this, SLOT(top_sql()));
    ///////////////////////////////////////////////////////
    ////////    V$...
    ///////////////////////////////////////////////////////
    vdollar = new QPopupMenu(this);
    menuBar()->insertItem( "&V$... views", vdollar);
        vdollar->insertItem( "Complete list", this, SLOT(v_complete_list()) );
	vdollar->insertSeparator();
        QPopupMenu * vdollarOld = new QPopupMenu(this);
        vdollar->insertItem("&Old style V$... menu",vdollarOld);
        vdollarOld->insertItem( "v$access", this, SLOT(v_access()) );
        vdollarOld->insertItem( "v$archive", this, SLOT(v_archive()) );
        vdollarOld->insertItem( "v$archived_log", this, SLOT(v_archived_log()) );
        vdollarOld->insertItem( "v$bgprocess", this, SLOT(v_bgprocess()) );
        vdollarOld->insertItem( "v$buffer_pool", this, SLOT(v_buffer_pool()) );
        vdollarOld->insertItem( "v$compatibility & v$compatseg", this, SLOT(v_compatibility_compatseg()) );
        vdollarOld->insertItem( "v$controlfile_record_section", this, SLOT(v_controlfile_record_section()) );
        vdollarOld->insertItem( "v$datafile_header", this, SLOT(v_datafile_header()) );
        vdollarOld->insertItem( "v$datafile", this, SLOT(v_datafile()) );
        vdollarOld->insertItem( "v$dblink", this, SLOT(v_dblink()) );
        vdollarOld->insertItem( "v$db_pipes", this, SLOT(v_db_pipes()) );
        vdollarOld->insertItem( "v$event_name", this, SLOT(v_event_name()) );
        vdollarOld->insertItem( "v$instance_recovery", this, SLOT(v_instance_recovery()) );
        vdollarOld->insertItem( "v$latch", this, SLOT(v_latch()) );
        vdollarOld->insertItem( "v$license", this, SLOT(v_license()) );
        vdollarOld->insertItem( "v$lock", this, SLOT(v_lock()) );
        vdollarOld->insertItem( "v$locked_object", this, SLOT(v_locked_object()) );
        vdollarOld->insertItem( "v$log_history", this, SLOT(v_log_history()) );
        vdollarOld->insertItem( "v$nls_database_parameter", this, SLOT(v_nls_parameters()) );
        vdollarOld->insertItem( "v$nls_instance_parameter", this, SLOT(v_nls_parameters()) );
        vdollarOld->insertItem( "v$nls_parameter", this, SLOT(v_nls_parameters()) );
        vdollarOld->insertItem( "v$nls_session_parameter", this, SLOT(v_nls_parameters()) );
        vdollarOld->insertItem( "v$nls_valid_values", this, SLOT(v_nls_valid_values()) );
        vdollarOld->insertItem( "v$object_dependency", this, SLOT(v_object_dependency()) );
        vdollarOld->insertItem( "v$obsolete_parameter", this, SLOT(v_obsolete_parameter()) );
        vdollarOld->insertItem( "v$option", this, SLOT(v_option()) );
        vdollarOld->insertItem( "v$parameter", this, SLOT(v_parameter()) );
        vdollarOld->insertItem( "v$pq_slave", this, SLOT(v_pq_slave()) );
        vdollarOld->insertItem( "v$pq_sysstat", this, SLOT(v_pq_sysstat()) );
        vdollarOld->insertItem( "v$process", this, SLOT(v_process()) );
        vdollarOld->insertItem( "v$pwfile_users", this, SLOT(v_pwfile_users()) );
        vdollarOld->insertItem( "v$reserved_words", this, SLOT(v_reserved_words()) );
        vdollarOld->insertItem( "v$resource_limit", this, SLOT(v_resource_limit()) );
        vdollarOld->insertItem( "v$rollstat", this, SLOT(v_rollstat()) );
        vdollarOld->insertItem( "v$rowcache_parent", this, SLOT(v_rowcache_parent()) );
        vdollarOld->insertItem( "v$rowcache_subordinate", this, SLOT(v_rowcache_subordinate()) );
        vdollarOld->insertItem( "v$session_longops", this, SLOT(v_session_longops()) );
        vdollarOld->insertItem( "v$sesstat", this, SLOT(v_sesstat()) );
        vdollarOld->insertItem( "v$session_wait", this, SLOT(v_session_wait()) );
        vdollarOld->insertItem( "v$shared_pool_reserved", this, SLOT(v_shared_pool_reserved()) );
        vdollarOld->insertItem( "v$sort_segment", this, SLOT(v_sort_segment()) );
        vdollarOld->insertItem( "v$sort_usage", this, SLOT(v_sort_usage()) );
        vdollarOld->insertItem( "v$sql_bind_data", this, SLOT(v_sql_bind_data()) );
        vdollarOld->insertItem( "v$sql_bind_metadata", this, SLOT(v_sql_bind_metadata()) );
        vdollarOld->insertItem( "v$sql_cursor", this, SLOT(v_sql_cursor()) );
        vdollarOld->insertItem( "v$sql_shared_memory", this, SLOT(v_sql_shared_memory()) );
        vdollarOld->insertItem( "v$subcache", this, SLOT(v_subcache()) );
        vdollarOld->insertItem( "v$sysstat", this, SLOT(v_sysstat()) );
        vdollarOld->insertItem( "v$system_event", this, SLOT(v_system_event()) );
        vdollarOld->insertItem( "v$tablespace", this, SLOT(v_tablespace()) );
        vdollarOld->insertItem( "v$tempfile", this, SLOT(v_tempfile()) );
        vdollarOld->insertItem( "v$transaction", this, SLOT(v_transaction()) );
        vdollarOld->insertItem( "v$waitstat", this, SLOT(v_waitstat()) );
	buildVDollarMenu();
    QPopupMenu * reports = new QPopupMenu(this);
    menuBar()->insertItem( "&Reports", reports );
        reports->insertItem( "&Tablespace sizes", this, SLOT(tbs_sizes()) );
        reports->insertItem( "&Extents left", this, SLOT(extents_left()) );
        reports->insertItem( "&Free space", this, SLOT(free_space()) );
        reports->insertItem( "F&ree space deficit", this, SLOT(free_space_deficit()) );
        reports->insertItem( "Invalid Objects", this, SLOT(invalid_objects()) );
    tools = new QPopupMenu(this);
    //QPopupMenu * tools = new QPopupMenu(this);
    menuBar()->insertItem( "&Tools", tools );
        tools->insertItem( "&Clear SQL cache", this, SLOT(clearSQLCache()) );
	userdefsqlIcon = QPixmap( userdefsql );
        tools->insertItem(userdefsqlIcon, "&User defined SQL", this, SLOT(UserDefinedSQL()) );
	tools->insertSeparator();
        tools->insertItem( "&SQL*Plus", this, SLOT(SQLPlus()) ,CTRL+Key_P);
        tools->insertItem( "SQL &Editor", this, SLOT(SQLEditor()) ,CTRL+Key_E);
        tools->insertItem( "&Adhoc SQL", this, SLOT(generateSQLCacheEntry()));
	tools->insertSeparator();
    	QPopupMenu * generalSQL = new QPopupMenu(this);
    	tools->insertItem("&Generate general SQL",generalSQL);
		generalSQL->insertItem("Create database...",this,SLOT(generate_database_create()));
#ifdef INHOUSE
		generalSQL->insertItem("crdbs config...",this,SLOT(generate_crdbs_config()));
#endif
	tools->insertSeparator();
        NIQItem=tools->insertItem( "No initial query", this, SLOT(toggleNoInitQuery()));
	tools->setItemChecked(NIQItem,false);
	
    windows= new QPopupMenu( this );
    windows->setCheckable( TRUE );
    connect( windows, SIGNAL( aboutToShow() ), this, SLOT( fillWindowsMenu() ) );
    menuBar()->insertItem( "&Windows", windows);                           
		
    ///////////////////////////////////////////////////////
    ////////    Help
    ///////////////////////////////////////////////////////
    QPopupMenu * help = new QPopupMenu(this);
    menuBar()->insertItem( "&Help", help );
        help->insertItem("About DBAComp...",this,SLOT(aboutDBAComp()));
        help->insertItem("About Qt...",this,SLOT(aboutQt()));
#ifdef INHOUSE
        help->insertItem("xmltest",this,SLOT(xmltest()));
#endif
} 

QPopupMenu* mdi::getActionMenu()
{
    return action;
}                                                                              

void mdi::replaceActionToolbar(QToolBar * tb)
{
	if (actionToolBar) 
	{
		actionToolBar->hide();
	}
	actionToolBar=tb;
	actionToolBar->show();
}

void mdi::replaceActionMenu(QPopupMenu * m)
{
    //if (action)
	// Test if the Action item is there
	// If we replace the action menu because we switch from one sheet to
	// another without deleting the sheet then the Action item is there and
	// must be deleted
	// If we replace the action menu because we switch from one sheet to 
	// another WITH deleting the sheet then there is no more action item
	// (it has been deleted in the sheet destructor [delete actionMenu
	// automatically deletes the item, Qt functionality])
	// and we don't need to delete it
    //if (menuBar()->text(menuBar()->idAt(1))=="&Action")
    if (ActionMenuID)
    {
        menuBar()->removeItem(ActionMenuID);
	ActionMenuID=0;
    }
    setActionMenu(m);
}

void mdi::setActionMenu(QPopupMenu* m)
{
	action=m;
	if (m)
	{
		// Calculate the index of the action menu in relation 
		// to the file menu (it must be behind the file menu)
		int ActionMenuIndex=menuBar()->indexOf(FileMenuID)+1;
		ActionMenuID=menuBar()->insertItem( "&Action", action,-1,ActionMenuIndex);
	}
	else
		ActionMenuID=0;
}

QString mdi::getOracleVersionString()
{
        QString oraversion_full;
        // Get version of Oracle database
        try
        {
		// This is the only SQL in DBAComp that is hard-coded
		// This is necessary as the result of this SQL is used
		// to setup the SQL cache and can thus not come from the 
		// SQL cache
		gConnObj->open("select v.banner from v$version v where upper(v.banner) like '%ORACLE%'");
		gConnObj->fetch();
		oraversion_full=gConnObj->getCol(0);
		gConnObj->close();
		return oraversion_full;
        }
        catch(oerr& e)
        {
		QMessageBox::critical( 0, "Database error when trying to get Oracle version.", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
		return "";
        }
}

/*
void mdi::setupSQLCache()
{
	gConfigFile->setGroup("CACHE");
    	const int maxCacheItems=(gConfigFile->readEntry("max_cache_items","50")).toInt();
	gConfigFile->setGroup("PATH");
    	//const QString cachefile=gConfigFile->readEntry("sql_cache_file","");
    	QString cachefile=gConfigFile->readEntry("sql_cache_file","");
	if (!QFile(cachefile).exists())
	{
		cerr << "sql_cache_file not set or incorrect. Trying local dbacomp.sql." << endl;
		cachefile=QString("dbacomp.sql");
	}

	QString oraversion_full=getOracleVersionString();
	if (oraversion_full!="")
	{
        	// Extract the 2 digit version of Oracle (e.g. 7.3, 8.0, 8.1, ...)
        	int verpos=oraversion_full.find(QRegExp("[0-9]\\.[0-9]\\.[0-9]\\.[0-9]\\.[0-9]",false,false));
        	QString oraversion_short=oraversion_full.mid(verpos,3);
		delete gSQLCache;
		gSQLCache=0;
        	gSQLCache = new SQLCache(cachefile,oraversion_short,maxCacheItems);
	}
	else
	{
		gMDI->setCaption("CONNECTED WITH COMPLICATIONS");
		QMessageBox::critical( 0, "SQL cache setup error", "Oracle version could not be determined",QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}
*/
void mdi::setupSQLCache()
{

	QString oraversion_full=getOracleVersionString();
	if (oraversion_full!="")
	{
        	// Extract the 2 digit version of Oracle (e.g. 7.3, 8.0, 8.1, ...)
        	int verpos=oraversion_full.find(QRegExp("[0-9]\\.[0-9]\\.[0-9]\\.[0-9]\\.[0-9]",false,false));
        	QString oraversion_short=oraversion_full.mid(verpos,3);

		// Get the max number of cache entries
		gConfigFile->setGroup("CACHE");
    		const int maxCacheItems=(gConfigFile->readEntry("max_cache_items","50")).toInt();

		// Get the dbacompXX.sql file
		gConfigFile->setGroup("PATH");
    		QString cachefile=gConfigFile->readEntry("sql_cache_file_prefix","");
		cachefile=cachefile+"."+oraversion_short+".sql";
		if (!QFile(cachefile).exists())
		{
			cerr << "sql_cache_file_prefix not set or incorrect. Trying local dbacomp."+oraversion_short+".sql" << endl;
			cachefile=QString("dbacomp."+oraversion_short+".sql");
		}
		delete gSQLCache;
		gSQLCache=0;
        	gSQLCache = new SQLCache(cachefile,oraversion_short,maxCacheItems);

	}
	else
	{
		gMDI->setCaption("CONNECTED WITH COMPLICATIONS");
		QMessageBox::critical( 0, "SQL cache setup error", "Oracle version could not be determined",QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

void mdi::setupMDITitle()
{
	if (gConnObj!=0)
        {
        	if (gTNSAlias=="")
                // local connection; we need to get the name from the database
                // so that we can show it in the title of the mdi window
                {
                	try
                        {
                        	gConnObj->open("select name from v$database");
                                gConnObj->fetch();
                                QString dbname=gConnObj->getCol(0);
                                gConnObj->close();
				mdiBaseTitle=dbname+" ("+getOracleVersionString()+")";
                                //gMDI->setCaption(dbname+" ("+getOracleVersionString()+")");
                                gMDI->setCaption(mdiBaseTitle);
                         }
                         catch(oerr& e)
                         {
				gMDI->setCaption("CONNECTED WITH COMPLICATIONS");
				QMessageBox::critical( 0, "Oracle error trying to determine database name", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
                         }
		}
                else
                {
			mdiBaseTitle=gTNSAlias+" ("+getOracleVersionString()+")";
                	gMDI->setCaption(mdiBaseTitle);
                	//gMDI->setCaption(gTNSAlias+" ("+getOracleVersionString()+")");
                }
	}
}

bool mdi::connectToDB()
{
	try
	{
//cerr << gUsername << gPassword << gTNSAlias << endl;
		statusBar()->message( "Connecting...");
		qApp->setOverrideCursor(waitCursor);
		if (!logoff()) return false;
		if (gUsername!="") // whole connect string provided
		{
			gConnObj = new qt_oraconnect("main",gUsername,gPassword,gTNSAlias);
		}
		else  // no username so OPS$ connection
		{
			if (gTNSAlias!="") // remote db
				gConnObj = new qt_oraconnect("main",0,0,gTNSAlias);
			else // local db
				gConnObj = new qt_oraconnect("main");
		}

		// Update ini file
        	gConfigFile->setGroup("CONNECTION");
        	gConfigFile->writeEntry("last_user",gUsername);
        	gConfigFile->writeEntry("last_db",gTNSAlias);

		// Check if current TNS alias is already in the list in the ini file
		// If not add it 
		gConfigFile->setGroup("CONNECTION");
		QString TNSAliasList=gConfigFile->readEntry("connect_strings","");
		if (TNSAliasList.find(gTNSAlias,0,false)==-1) 
		{
			TNSAliasList+=","+gTNSAlias;
			 gConfigFile->writeEntry("connect_strings",TNSAliasList);
		}
		gConfigFile->write();
		qApp->restoreOverrideCursor();
		statusBar()->clear();
		return true;
	}
	catch(oerr& e)
	{
		// Unwind the waitCursors that may have been set elsewhere
		while (qApp->overrideCursor()) qApp->restoreOverrideCursor();
		statusBar()->clear();
		QMessageBox::critical( 0, "Database connection error", QString("Oracle error: ") + ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
		logoff();
		return false;
	}
}

void mdi::customizeSession()
{
	try
	{
		QString sql=gSQLCache->getSQL("alter_session");
		if (sql=="")
			QMessageBox::warning( 0, "Error during session customisation", QString("The following SQL could not be found: alter_session!\nDBAComp continues with the default session settings."),QMessageBox::Ok | QMessageBox::Default,0,0);
		else
		{
			gConnObj->open(sql);
			gConnObj->close();
		}
	}
	catch (oerr& e)
	{
		// This is not a fatal error it simply prevents 
		// the session to be customised
		QMessageBox::warning( 0, "Error during session customisation", QString("Error in 'alter_session' SQL: ") + ((qt_oerr&)e).getMsg()+"\nDBAComp continues with the default session settings.",QMessageBox::Ok | QMessageBox::Default,0,0);
	}
}

void mdi::database_overview()
{
	w_db_overview *w = new w_db_overview();
	//DBTab *w = new DBTab(0,"Database overview");
//	w->addFormTab("v_database","Database",1,190,130);
//	w->show();
}

void mdi::dba_users()
{
    	w_dba_users *w = new w_dba_users();
}

void mdi::dba_roles()
{
    	w_dba_roles *w = new w_dba_roles();
}

void mdi::dba_profiles()
{
    	DBList *w = new DBList("dba_profiles",0,"DBA_PROFILES");
	w->show();
}

void mdi::dba_tablespaces()
{
    	w_dba_tablespaces *w = new w_dba_tablespaces();
}

void mdi::dba_rollback_segs()
{
        w_dba_rollback_segs *w = new w_dba_rollback_segs();
}

void mdi::dba_data_files()
{
        w_dba_data_files *w = new w_dba_data_files();
}
void mdi::v_log()
{
        w_v_log *w = new w_v_log();
}

void mdi::v_controlfile()
{
    	DBList *w = new DBList("v_controlfile",0,"Control files");
	w->show();
}

void mdi::all_objects()
{
        w_all_objects *w = new w_all_objects();
}                                                                                            

void mdi::perf_indicators()
{
        w_performance *w = new w_performance();
}

void mdi::hot_files()
{
        DBList *w = new DBList("hot_files",0,"Hot files");
        w->show();
}

void mdi::top_sql()
{
        DBList *w = new DBList("top_sql",0,"Top SQL");
        w->show();
}

void mdi::v_access()
{
        DBList *w = new DBList("v_access",0,"V$ACCESS");
        w->show();
}
void mdi::v_archive()
{
        DBList *w = new DBList("v_archive",0,"V$ARCHIVE");
        w->show();
}
void mdi::v_buffer_pool()
{
        DBList *w = new DBList("v_buffer_pool",0,"V$BUFFER_POOL");
        w->show();
}
void mdi::v_bgprocess()
{
        DBList *w = new DBList("v_bgprocess",0,"V$BGPROCESS");
        w->show();
}
void mdi::v_compatibility_compatseg()
{
        DBList *w = new DBList("v_compatibility_compatseg",0,"V$COMPATIBILITY and V$COMPATSEG");
        w->show();
}

void mdi::v_controlfile_record_section()
{
        DBList *w = new DBList("v_controlfile_record_section",0,"V$CONTROL_FILE_RECORD_SECTION");
        w->show();
}

void mdi::v_datafile_header()
{
        DBList *w = new DBList("v_datafile_header",0,"V$DATAFILE_HEADER");
        w->show();
}
void mdi::v_dblink()
{
        DBList *w = new DBList("v_dblink",0,"V$DBLINK");
        w->show();
}
void mdi::v_db_pipes()
{
        DBList *w = new DBList("v_db_pipes",0,"V$DB_PIPES");
        w->show();
}
void mdi::v_event_name()
{
        DBList *w = new DBList("v_event_name",0,"V$EVENT_NAME");
        w->show();
}
void mdi::v_instance_recovery()
{
        DBList *w = new DBList("v_instance_recovery",0,"V$INSTANCE_RECOVERY");
        w->show();
}
void mdi::v_latch()
{
        DBList *w = new DBList("v_latch",0,"V$LATCH");
        w->show();
}
void mdi::v_license()
{
        DBList *w = new DBList("v_license",0,"V$LICENSE");
        w->show();
}

void mdi::v_lock()
{
        DBList *w = new DBList("v_lock",0,"V$LOCK");
        w->show();
}

void mdi::locktree()
{
        w_locktree  *w = new w_locktree();
}

void mdi::v_locked_object()
{
        DBList *w = new DBList("v_locked_object",0,"V$LOCKED_OBJECT");
        w->show();
}
void mdi::v_log_history()
{
        DBList *w = new DBList("v_log_history",0,"V$LOG_HISTORY");
        w->show();
}
void mdi::v_nls_parameters()
{
        DBList *w = new DBList("v_nls_parameters",0,"V$NLS_PARAMETERS");
        w->show();
}
void mdi::v_nls_database_parameters()
{
        DBList *w = new DBList("v_nls_database_parameters",0,"V$NLS_DATABASE_PARAMETERS");
        w->show();
}
void mdi::v_nls_instance_parameters()
{
        DBList *w = new DBList("v_nls_instance_parameters",0,"V$NLS_INSTANCE_PARAMETERS");
        w->show();
}
void mdi::v_nls_session_parameters()
{
        DBList *w = new DBList("v_nls_session_parameters",0,"V$NLS_SESSION_PARAMETERS");
        w->show();
}
void mdi::v_complete_list()
{
        //DBList *w = new DBList("v_complete_list",0,"Complete list of v$ views");
        //w->show();
	new w_v_complete_list();
}
void mdi::v_nls_valid_values()
{
        DBList *w = new DBList("v_nls_valid_values",0,"V$NLS_VALID_VALUES");
        w->show();
}
void mdi::v_option()
{
        DBList *w = new DBList("v_option",0,"V$OPTION");
        w->show();
}

void mdi::v_parameter()
{
        DBList *w = new DBList("v_parameter",0,"V$PARAMETER",false,false);
        w->show();
}
void mdi::v_pq_slave()
{
        DBList *w = new DBList("v_pq_slave",0,"V$PQ_SLAVE");
        w->show();
}
void mdi::v_pq_sysstat()
{
        DBList *w = new DBList("v_pq_sysstat",0,"V$PQ_SYSSTAT");
        w->show();
}

void mdi::v_process()
{
        DBList *w = new DBList("v_process",0,"V$PROCESS");
        w->show();
}

void mdi::v_shared_pool_reserved()
{
        DBList *w = new DBList("v_shared_pool_reserved",0,"V$SHARED_POOL_RESERVED");
        w->show();
}
void mdi::v_pwfile_users()
{
        DBList *w = new DBList("v_pwfile_users",0,"V$PWFILE_USERS");
        w->show();
}
void mdi::v_resource_limit()
{
        DBList *w = new DBList("v_resource_limit",0,"V$RESOURCE_LIMIT");
        w->show();
}
void mdi::v_rollstat()
{
        DBList *w = new DBList("v_rollstat",0,"V$ROLLSTAT");
        w->show();
}
void mdi::v_session_longops()
{
        DBList *w = new DBList("v_session_longops",0,"V$SESSION_LONGOPS");
        w->show();
}
void mdi::v_sesstat()
{
        DBList *w = new DBList("v_sesstat",0,"V$SESSTAT");
        w->show();
}
void mdi::v_session_wait()
{
        DBList *w = new DBList("v_session_wait",0,"V$SESSION_WAIT");
        w->show();
}
void mdi::v_sort_segment()
{
        DBList *w = new DBList("v_sort_segment",0,"V$SORT_SEGMENT");
        w->show();
}
void mdi::v_sort_usage()
{
        DBList *w = new DBList("v_sort_usage",0,"V$SORT_USAGE");
        w->show();
}
void mdi::v_sql_bind_data()
{
        DBList *w = new DBList("v_sql_bind_data",0,"V$SQL_BIND_DATA");
        w->show();
}
void mdi::v_sql_bind_metadata()
{
        DBList *w = new DBList("v_sql_bind_metadata",0,"V$SQL_BIND_METADATA");
        w->show();
}
void mdi::v_sql_cursor()
{
        DBList *w = new DBList("v_sql_cursor",0,"V$SQL_CURSOR");
        w->show();
}
void mdi::v_sql_shared_memory()
{
        DBList *w = new DBList("v_sql_shared_memory",0,"V$SQL_SHARED_MEMORY");
        w->show();
}
void mdi::v_subcache()
{
        DBList *w = new DBList("v_subcache",0,"V$SUBCACHE");
        w->show();
}
void mdi::v_object_dependency()
{
        DBList *w = new DBList("v_object_dependency",0,"V$OBJECT_DEPENDENCY");
        w->show();
}
void mdi::v_obsolete_parameter()
{
        DBList *w = new DBList("v_obsolete_parameter",0,"V$OBSOLETE_PARAMETER");
        w->show();
}
void mdi::v_reserved_words()
{
        DBList *w = new DBList("v_reserved_words",0,"V$RESERVED_WORDS");
        w->show();
}
void mdi::v_rowcache_parent()
{
        DBList *w = new DBList("v_rowcache_parent",0,"V$ROWCACHE_PARENT");
        w->show();
}
void mdi::v_rowcache_subordinate()
{
        DBList *w = new DBList("v_rowcache_subordinate",0,"V$ROWCACHE_SUBORDINATE");
        w->show();
}
void mdi::v_tempfile()
{
        DBList *w = new DBList("v_tempfile",0,"V$TEMPFILE");
        w->show();
}
void mdi::v_sysstat()
{
        DBList *w = new DBList("v_sysstat",0,"V$SYSSTAT");
        w->show();
}
void mdi::v_system_event()
{
        DBList *w = new DBList("v_system_event",0,"V$SYSTEM_EVENT");
        w->show();
}
void mdi::v_tablespace()
{
        DBList *w = new DBList("v_tablespace",0,"V$TABLESPACE");
        w->show();
}
void mdi::v_transaction()
{
        DBList *w = new DBList("v_transaction",0,"V$TRANSACTION");
        w->show();
}
void mdi::v_waitstat()
{
        DBList *w = new DBList("v_waitstat",0,"V$WAITSTAT");
        w->show();
}

void mdi::v_session()
{
    	//DBList *w = new DBList("v_session",0,"V$SESSION");
	//w->show();
	new w_v_session();
}

void mdi::v_datafile()
{
    	DBList *w = new DBList("v_datafile",0,"V$DATAFILE");
	w->show();
}

bool mdi::logoff()
{
	if (gMDI->deleteAllSheets())
	{
		delete gSQLCache;
		gSQLCache=0;
		try
		{
			delete gConnObj;
			gConnObj=0;
		}
       	 	catch(oerr& e)
       	 	{
			// Unwind the waitCursors that may have been set elsewhere
			while (qApp->overrideCursor()) qApp->restoreOverrideCursor();
			statusBar()->clear();
       	         	QMessageBox::critical( 0, "Database error when logging off.", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
       	 	}
		gConnObj=0;
		gMDI->setCaption("(NOT CONNECTED)");
		changeMenuState(false);
		return true;
	}
	else return false;
}

/* Original
void mdi::logoff()
{
	delete gSQLCache;
	gSQLCache=0;
	try
	{
		delete gConnObj;
		gConnObj=0;
	}
        catch(oerr& e)
        {
                QMessageBox::critical( 0, "Database error when logging off.", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
        }
	gConnObj=0;
	gMDI->setCaption("(NOT CONNECTED)");
	gMDI->deleteAllSheets();
	changeMenuState(false);
}
*/

void mdi::postInteractiveLogon()
{
	gCommandLineArg=""; // to make sure it does an interactive logon
	QEvent* logonEvent = new QEvent(QEvent::User);
	qApp->postEvent(gMDI,logonEvent);
}

void mdi::interactive_logon()
{
	// Logon dialog
	if (gUsername.isEmpty())
	{
		gConfigFile->setGroup("CONNECTION");
		gUsername=gConfigFile->readEntry("last_user","");
	}
    	if (gTNSAlias.isEmpty())
    	{
        	gConfigFile->setGroup("CONNECTION");
        	gTNSAlias=gConfigFile->readEntry("last_db","");
    	}                                                                           	gConfigFile->setGroup("CONNECTION");
	QString TNSAliasList=gConfigFile->readEntry("connect_strings","");
	LogonDlg *ld = new LogonDlg(this,"logondlg",gUsername,gPassword,gTNSAlias,TNSAliasList);
	bool Connected=false;
	while (!Connected)
	{
        	if (ld->exec())
       	 	{
               	 	gUsername=ld->getUsername();
                	gPassword=ld->getPassword();
                	gTNSAlias=ld->getTNSAlias();
			Connected=connectToDB();
        	}
		else break;
        }
        delete ld;
	ld=0;
	if (Connected) 
	{
		setupSQLCache();
		registerApplication();
		customizeSession();
		setupMDITitle();
		changeMenuState(true);
	}
}

void mdi::changeMenuState(bool enabled)
{
	menuBar()->setItemEnabled(menuBar()->idAt(1),enabled);
	menuBar()->setItemEnabled(menuBar()->idAt(2),enabled);
	menuBar()->setItemEnabled(menuBar()->idAt(3),enabled);
	menuBar()->setItemEnabled(menuBar()->idAt(4),enabled);
	menuBar()->setItemEnabled(menuBar()->idAt(5),enabled);
	menuBar()->setItemEnabled(menuBar()->idAt(6),enabled);

    QObjectList *list = ((QObject *)ToolBar)->queryList("QToolButton");
    QObjectListIt it( *list );          // iterate over the buttons
    QObject * obj;
    while ( (obj=it.current()) != 0 )
    { // for each found object...
        if (((QString)((QToolButton *)obj)->name())=="Users"
        ||  ((QString)((QToolButton *)obj)->name())=="Tablespaces"
        ||  ((QString)((QToolButton *)obj)->name())=="Rollback segs"
        ||  ((QString)((QToolButton *)obj)->name())=="Datafiles"
        ||  ((QString)((QToolButton *)obj)->name())=="Sessions"
        ||  ((QString)((QToolButton *)obj)->name())=="User defined SQL")
        {
            ((QToolButton *)obj)->setEnabled(enabled);
        }
        ++it;
    }                                                                                                             
}


void mdi::clearSQLCache()
{
	if (gSQLCache) gSQLCache->clear();
	else QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
}

void mdi::aboutDBAComp()
{
    QMessageBox::about( this, "About DBAComp", 
QString("                      DBAComp version ")+gVersion+QString("\n\n")+

QString("Copyright (C) 2000-2003 by Torsten FASSIAN (torsten@internet.lu).\n\n")+

QString("DBAComp comes with ABSOLUTELY NO WARRANTY. This is free software\n")+
QString("and you are welcome to redistribute it under the terms of the GNU General\n")+
QString("Public License. You can download it from homepage.internet.lu/torsten.\n\n")+

QString("                         See 'LICENSE' for details. "));
}

void mdi::aboutQt()
{
    QMessageBox::aboutQt( this, "About Qt" );
}

void mdi::setActionToolBar(QToolBar *tb)
{
	actionToolBar=tb;
}

void mdi::fillWindowsMenu()
{
    windows->clear();
    int cascadeId = windows->insertItem("&Cascade", ws, SLOT(cascade() ) );
    int tileId = windows->insertItem("&Tile", ws, SLOT(tile() ) );
    if ( ws->windowList().isEmpty() ) 
	{
    	windows->setItemEnabled( cascadeId, FALSE );
    	windows->setItemEnabled( tileId, FALSE );
    }
    windows->insertSeparator();
    QWidgetList windowsList = ws->windowList();
    for ( int i = 0; i < int(windowsList.count()); ++i ) 
	{
    	int id = windows->insertItem(windowsList.at(i)->caption(), this, SLOT( windowsMenuActivated( int ) ) );
    	windows->setItemParameter( id, i );
    	windows->setItemChecked( id, ws->activeWindow() == windowsList.at(i) );
    }                                                                                                                 
}                                                                               

void mdi::windowsMenuActivated( int id )
{
    Sheet* w = ((Sheet*)ws->windowList().at( id ));
    if ( w ) w->setFocus();
}                                                                               

void mdi::UserDefinedSQL()
{
	UserSQLList * w = new UserSQLList();
	w->show();
}

void mdi::SQLPlus()
{
	gConfigFile->setGroup("MISC");
	QString sqlplus=gConfigFile->readEntry("sqlplus","");
	QString connectString=gUsername+"/"+gPassword;
	if (!gTNSAlias.isEmpty()) connectString=connectString+"@"+gTNSAlias;

	sqlplus.replace(QRegExp("<SPC>")," ");
	sqlplus.replace(QRegExp("<USERNAME>"),gUsername);
	sqlplus.replace(QRegExp("<TNSALIAS>"),gTNSAlias);
	sqlplus.replace(QRegExp("<CONNECTSTRING>"),connectString);
	system(sqlplus);
}

void mdi::SQLEditor()
{
	Editor *e = new Editor(this, "Editor");
	e->show();
}

void mdi::generateSQLCacheEntry()
{
	(new GenerateSQLCacheEntry())->show();	
}

void mdi::removeActionItem()
{
	if (ActionMenuID) menuBar()->removeItem(ActionMenuID);
}

void mdi::v_backup()
{
    	DBList *w = new DBList("v_backup",0,"V$BACKUP");
	w->show();
}

void mdi::v_backup_v_datafile()
{
    	DBList *w = new DBList("v_backup_v_datafile",0,"V$BACKUP and V$DATAFILE");
	w->show();
}
void mdi::v_backup_corruption()
{
    	DBList *w = new DBList("v_backup_corruption",0,"V$BACKUP_CORRUPTION");
	w->show();
}
void mdi::v_backup_datafile()
{
    	DBList *w = new DBList("v_backup_datafile",0,"V$BACKUP_DATAFILE");
	w->show();
}
void mdi::v_backup_device()
{
    	DBList *w = new DBList("v_backup_device",0,"V$BACKUP_DEVICE");
	w->show();
}
void mdi::v_backup_piece()
{
    	DBList *w = new DBList("v_backup_piece",0,"V$BACKUP_PIECE");
	w->show();
}
void mdi::v_backup_redolog()
{
    	DBList *w = new DBList("v_backup_redolog",0,"V$BACKUP_REDOLOG");
	w->show();
}
void mdi::v_backup_set()
{
    	DBList *w = new DBList("v_backup_set",0,"V$BACKUP_SET");
	w->show();
}

void mdi::v_recover_file()
{
    	DBList *w = new DBList("v_recover_file",0,"V$RECOVER_FILE");
	w->show();
}
void mdi::v_recovery_log()
{
    	DBList *w = new DBList("v_recovery_log",0,"V$RECOVERY_LOG");
	w->show();
}
void mdi::datafile_recovery_status()
{
    	DBList *w = new DBList("datafile_recovery_status",0,"Datafile recovery status");
	w->show();
}
void mdi::v_recovery_file_status()
{
    	DBList *w = new DBList("v_recovery_file_status",0,"V$RECOVERY_FILE_STATUS");
	w->show();
}
void mdi::v_recovery_status()
{
    	DBList *w = new DBList("v_recovery_status",0,"V$RECOVERY_STATUS");
	w->show();
}
void mdi::instance_overview()
{
	DBTab *w = new DBTab(0,"Instance overview");
	w->addFormTab("v_instance","Overview",1,190,135);
	w->addFormTab("v_thread","Thread",1,160,135);
	w->addGridTab("dba_jobs_running","Jobs running");
	w->show();
}
void mdi::sga()
{
	DBTab *w = new DBTab(0,"SGA");
	w->addGridTab("v_sga","SGA");
	w->addGridTab("v_sgastat","Statistics");
	w->addGridTab("v_bh","Buffer cache");
	w->addGridTab("v_sqlarea","Shared SQL");
	w->addFormTab("v_shared_pool_reserved","Shared Pool Reserved",1,260,260);
	w->addGridTab("v_rowcache","Row cache");
	w->addGridTab("v_librarycache","Library cache");
	w->addGridTab("v_db_object_cache","Object cache");
	w->show();
}
void mdi::sessions_processes()
{
    	DBList *w = new DBList("v_session_v_process",0,"Sessions and Processes");
	w->show();
}
void mdi::sessions_transactions()
{
    	DBList *w = new DBList("v_session_v_transaction",0,"Sessions and Transactions");
	w->show();
}
void mdi::locks()
{
    	DBList *w = new DBList("locks",0,"Locks");
	w->show();
}
void mdi::audit_options()
{
    	DBList *w = new DBList("dba_stmt_audit_opts",0,"Audit options");
	w->show();
}
void mdi::object_audit_options()
{
    	DBList *w = new DBList("dba_obj_audit_opts",0,"Object audit options");
	w->show();
}
void mdi::default_audit_options()
{
    	DBList *w = new DBList("all_def_audit_opts",0,"Default audit options");
	w->show();
}
void mdi::audit_trail()
{
    	DBList *w = new DBList("dba_audit_trail",0,"Audit trail");
	w->show();
}
void mdi::audit_sessions()
{
    	DBList *w = new DBList("dba_audit_session",0,"Audit sessions");
	w->show();
}
void mdi::audit_objects()
{
    	DBList *w = new DBList("dba_audit_object",0,"Audit objects");
	w->show();
}

void mdi::mts()
{
	DBTab *w = new DBTab(0,"MTS overview");
	w->addFormTab("v_mts","Overview",1,190,135);
	w->addGridTab("v_dispatcher","Dispatchers");
	w->addGridTab("v_shared_server","Shared servers");
	w->addGridTab("v_queue","Queues");
	w->show();
}

void mdi::tbs_sizes()
{
    	new w_dba_tablespaces("tbs_sizes","Tablespace sizes");
    	//DBList *w = new DBList("tbs_sizes",0,"Tablespace sizes");
	//w->show();
}

void mdi::extents_left()
{
    	DBList *w = new DBList("extents_left",0,"Extents left");
	w->show();
}

void mdi::free_space()
{
    	//w_free_space *w = new w_free_space();
    	new w_dba_tablespaces("free_space","Free Space");
}

void mdi::free_space_deficit()
{
    	DBList *w = new DBList("free_space_deficit",0,"Free space deficit");
	w->show();
}

void mdi::invalid_objects()
{
    	w_invalid_objects *w = new w_invalid_objects();
	w->show();
}

void mdi::v_archived_log()
{
    	DBList *w = new DBList("v_archived_log",0,"V$ARCHIVED_LOG");
	w->show();
}

void mdi::generate_crdbs_config()
{
        generate_sql("crdbs_config.sql","crdbs_config");
}

void mdi::generate_database_create()
{
        generate_sql("database_create.sql","database_create");
}

void mdi::generate_sql(const QString& fn, const QString& what)
{
        QString filename;
        bool    show_script;

        gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");

        ScriptDlg *sd = new ScriptDlg(path+"/"+fn,this,QString::null,false);
        if (sd->exec())
        {
		qApp->setOverrideCursor(waitCursor);
                filename=sd->getFilename();
                show_script=sd->getShow();
                delete sd;
		sd=0;

                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream t(&f);
                        int retval=0;
                        if (what=="crdbs_config")
                        	retval=GenSQL::CrdbsConfig(t);
                        else if (what=="database_create")
                        	retval=GenSQL::DatabaseCreate(t);
                        if (retval==-1)
                        {
                                QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                f.close();
                                return;
                        }
                        else if (retval==-2)
                        {
                                QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                f.close();
                                return;
                        }
                        else if (retval==-3)
                        {
                                QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                f.close();
                                return;
                        }
                }
                f.close();

                if (show_script)
                {
                        Editor * ed = new Editor(filename);
                        ed->show();
                }
		qApp->restoreOverrideCursor();
        }
        else
        {
                delete sd;
		sd=0;
        }
}

void mdi::registerApplication()
{
        try
        {
                QString sql=gSQLCache->getSQL("register_application");
                if (sql=="")
                        QMessageBox::warning( 0, "Error during application registration", QString("The following SQL could not be found: register_application!\nDBAComp continues without registering."),QMessageBox::Ok | QMessageBox::Default,0,0);
                else
                {
			gConnObj->open(sql,qt_OraParm(gVersion));
			gConnObj->close();
                }
        }
        catch (oerr& e)
        {
                // This is not a fatal error it simply prevents
                // the application to be registered
                QMessageBox::warning( 0, "Error during application registration", QString("Error in 'alter_session' SQL: ") + ((qt_oerr&)e).getMsg()+"\nDBAComp continues with registering.",QMessageBox::Ok | QMessageBox::Default,0,0);
        }
}

Sheet* mdi::getActiveSheet()
{
	return(activeSheet);
}

void mdi::newDBAComp()
{
	system("dbacomp&");
}

void mdi::quitDBAComp()
{
	//qApp->quit();
	//if (logoff()) qApp->quit();
	close();
}

void mdi::closeEvent( QCloseEvent* ce )
{
	if (logoff()) 
		ce->accept();
	else
		ce->ignore();
	
}

QDomElement* mdi::findXmlWindow(const QString& w)
{

        QDomDocument doc( "dbacomp_structure" );
        QFile f( "dbacomp.xml" );
        if ( !f.open( IO_ReadOnly ) )
                return 0;
        if ( !doc.setContent( &f ) )
        {
                f.close();
                return 0;
        }
        f.close();

        QDomElement docElem = doc.documentElement();

        QDomNode n = docElem.firstChild();
        // look for the <windows> tag
        while( !n.isNull() )
        {
                QDomElement e = n.toElement(); // try to convert the node to an element.
                if( !e.isNull() )
                { // the node was really an element.
                        if (e.tagName()=="windows")
                        {
                                n=e.firstChild();
                                break;
                        }
                }
                n = n.nextSibling();
        }
        // go through the windows
        while( !n.isNull() )
        {
                QDomElement e = n.toElement(); // try to convert the node to an element.
                if( !e.isNull() )
                { // the node was really an element.
                        if (e.tagName()=="window")
                        {
                                if (e.attribute("name")==w)
					return new QDomElement(e);
                        }
                }
                n = n.nextSibling();
        }
	return 0;
}


void mdi::xmltest()
{

	QDomDocument doc( "dbacomp_structure" );
  	QFile f( "dbacomp.xml" );
  	if ( !f.open( IO_ReadOnly ) )
      		return;
  	if ( !doc.setContent( &f ) ) 
	{
      		f.close();
      		return;
  	}
  	f.close();

  	QDomElement docElem = doc.documentElement();

  	QDomNode n = docElem.firstChild();
	// look for the <menus> tag
  	while( !n.isNull() ) 
	{
      		QDomElement e = n.toElement(); // try to convert the node to an element.
      		if( !e.isNull() ) 
		{ // the node was really an element.
          		if (e.tagName()=="menus") 
			{
				n=e.firstChild();
				break;
			}
      		}
      		n = n.nextSibling();
  	}
	// go through the menus
	build_menus(n,menuBar());
}

void mdi::build_menus(QDomNode n, QMenuData* menu)
{
	while( !n.isNull() )
        {
                QDomElement e = n.toElement(); // try to convert the node to an element.
                if( !e.isNull() )
                { // the node was really an element.
			cerr << e.tagName() << endl;
			cerr << e.attribute("name","attrib does not exist") << endl;
			if (e.tagName()=="menu")
			{
				QPopupMenu * submenu= new QPopupMenu(this);
				menu->insertItem(e.attribute("caption","attrib does not exist"), submenu);
				build_menus(e.firstChild(),submenu);
			}
			else if (e.tagName()=="menuitem")
			{
				//menu->insertItem( e.attribute("caption","attrib does not exist"));
				cerr << "id: " << e.attribute("id").toInt() << endl;
				//menu->insertItem(e.attribute("caption"), this, SLOT(windowFactory(int)), e.attribute("id").toInt());
				int retval=menu->insertItem(e.attribute("caption"), e.attribute("id").toInt());
				cerr << "retval: " << retval << endl;
				menu->connectItem(retval,this,SLOT(windowFactory(int)));
			}
                }
                n = n.nextSibling();
        }
}

void mdi::windowFactory(int menuid)
{
	DBList *w = new DBList(findXmlWindow("w_dba_users"));
        w->show();

}

void mdi::toggleNoInitQuery()
{
        if (noInitQuery)
        {
		noInitQuery=false;
		//gMDI->setCaption(mdiBaseTitle);
		statusBar()->message(QString::null);
        }
        else
        {
		noInitQuery=true;
		//gMDI->setCaption(mdiBaseTitle+" [no initial query]");
		statusBar()->message("No initial query mode");
        }
	tools->setItemChecked(NIQItem,noInitQuery);
}

const bool& mdi::getNoInitQuery() const
{
	return noInitQuery;
}

void mdi:: callVDollar(int name)
{
	QString *vDollarView=vdollarMenuCache->find(name);
	QString query="select * from "+QString(*vDollarView);
        DBList *v_view=new DBList(query,0,QString(*vDollarView),true);
        v_view->show();
}

void mdi::addToVDollarMenu(QString name)
{
	// first check if the v$ view is not already in the menu
        QIntDictIterator<QString> it(*vdollarMenuCache); 
	bool doIt=true;
	while ( it.current() ) 
	{
		if (*(QString*)it.current()==name)
		{
			doIt=false;
			++it;
			break;
		}
		++it;
	}
	// insert into the menu
	if (doIt)
	{
		int newID=vdollar->insertItem( name, this, SLOT(callVDollar(int)));
		vdollar->setItemParameter(newID,newID);
		vdollarMenuCache->resize(vdollarMenuCache->size()+1);
		vdollarMenuCache->insert(newID,new QString(name));
	}
}

void mdi::buildVDollarMenu()
{
	vdollar->insertSeparator();
	gConfigFile->setGroup("VDOLLAR");
	QString vdollar_menu_item_list=gConfigFile->readEntry("menu_items","");
	QStringList sortedList=QStringList::split(",",vdollar_menu_item_list,false);
	sortedList.sort();

	QStringList::Iterator it;

	for (it=sortedList.begin();it!=sortedList.end();++it)
	{
		addToVDollarMenu((*it).stripWhiteSpace().upper());
	}
	vdollar->insertSeparator();
	
	
}

void mdi::saveVDollarMenu()
{
// Save the vdollar menu to the ini file
        QIntDictIterator<QString> it(*vdollarMenuCache);
	QString menu_items;
        while ( it.current() )
        {
		menu_items=menu_items+(*(QString*)it.current());
                ++it;
		if (it.current()) menu_items=menu_items+',';
        }
	gConfigFile->setGroup("VDOLLAR");
	gConfigFile->writeEntry("menu_items",menu_items);
	gConfigFile->write();
}
