/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "mledatadisplay.h" 
#include "row.h" 
#include <iostream.h> 
#include "qt_oraconnect.h"
#include <qheader.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qmenudata.h>
#include <qaccel.h>
#include <qnamespace.h>
// For qt3
#include <qpainter.h>

//MLEDataDisplay::MLEDataDisplay(qt_oraconnect * cObj, const char * sql, 
MLEDataDisplay::MLEDataDisplay(qt_oraconnect * cObj, const QString& sql, 
			       QWidget * parent, const QString& name,
			       //QWidget * parent, const char * name,
			       bool headers, QString line_sep, 
			       QString col_sep, QString head_sep)
	: MultiLineEdit(parent,name), DataDisplay(cObj,sql), 
	  ShowColHeaders(headers), LineSeparator(line_sep), 
	  ColSeparator(col_sep), HeadSeparator(head_sep)
{
}

long MLEDataDisplay::retrieve(const ObjParm * parm)
{
   try
   {
	bool firstTime=false;
	if (parm)
	{
                // Copy this parm to an qt_OraParm
                qt_OraParm op(parm->ParmCount());
                for (int i=0;i<parm->ParmCount();i++)
		{
                        op.addParm(parm->getParm(i));
		}
		getConnObj()->open(getSQL(),op);
	}
	else
		getConnObj()->open(getSQL());
	// Number of columns
	int ncols=getConnObj()->getCols();
	QString text; // The whole text to be shown in the MLE
	// show the col headers if required
	if (ShowColHeaders)
	{
		// loop through the col names
		for (int i=0;i<ncols;i++)
		{
			text+=getConnObj()->getColName(i);
			if (i<ncols-1) text+=ColSeparator; 
		}
		text+=HeadSeparator;
	}
	// populate column list used for sort and filter dialogs
	// if not already done
	if (getColList().isEmpty()) 
	{
                // The fact that there are no columns probably means that this
 	        // is the first time that retrieve is called
               	firstTime=true;

		for (int j=0;j<ncols;j++) addToColList(getConnObj()->getColName(j));
	}
        // if this is not the initial query (at dd creation time)
        // and if a retrieve should be done at the initial query
        // then fetch the rows
        // else don't fetch the rows
        if (!(firstTime && getNoInitQuery()))
        {
        	// get the data from the database
        	for (;getConnObj()->fetch() == 0;)
        	{
                	// show the values of all the cols of this record
                	for (int i=0;i<ncols;i++)
                	{
				text+=getConnObj()->getCol(i);
				if (i<ncols-1) text+=ColSeparator; 
                	}
			text+=LineSeparator;
		}
		if (!getLogMode()) setText(text);
		else append(text);
	}
	long noRows=getConnObj()->count();
	getConnObj()->close();

	return(noRows);
   }

   catch(oerr& e)
   {
	SQLErrorEvent(((qt_oerr&)e).getMsg());
	return -1;
   }
}

void MLEDataDisplay::SQLErrorEvent(const QString& msg)
{
	emit SQLError(msg);
	// Unwind the waitCursors that may have been set elsewhere
	while (qApp->overrideCursor()) qApp->restoreOverrideCursor();
	QMessageBox::critical( 0, "SQL error", QString("Oracle error: ") + msg,QMessageBox::Ok | QMessageBox::Default,0,0);
}

void MLEDataDisplay::print()
{
	printData();
}

void MLEDataDisplay::save()
{
	saveData();
}

QPopupMenu* MLEDataDisplay::getMenuAddition()
{
	// no menu addition yet
	if (DataDisplay::getMenuAddition()==0)
	{
		// create one
		QPopupMenu* menu=new QPopupMenu(this);
		menu->insertItem("Find...",(MultiLineEdit*)this,SLOT(findDialog()),CTRL+Key_F);
		menu->insertItem("Replace...",(MultiLineEdit*)this,SLOT(replaceDialog()),CTRL+Key_R);
		DataDisplay::setMenuAddition(menu);
	}
        return DataDisplay::getMenuAddition();
}
