/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <qpalette.h>
#include <iostream.h>
#include <qheader.h>
#include <qregexp.h>
#include <stdlib.h>
#include <errno.h>
#include "row.h"
#include "general.h"
// For qt3
#include <qpainter.h>

row::row(QListView * parent, bool ana)
	: QListViewItem( parent ), cells(0),allowNumberAlign(ana)
{
}

row::row(QListView * parent, row *after, bool ana)
	: QListViewItem( parent, after ), cells(0),allowNumberAlign(ana)
{
}

row::~row()
{
	delete [] cells;
	cells=0;
}

void row::createCells(int pNoCells)
{
	NoCells = pNoCells;
	cells = new cell[pNoCells];
}

void row::setText ( int column, const QString & text )
{
	//cells[column-1].setContent(text);
	// changed for QT2.1
	cells[column].setContent(text);
}

QString row::text(int column) const
{
	if (cells)
		return cells[column].getContent();
	else
		return 0;
}	

/* This is the way it had to be in QT2.0
void row::paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int align )
{
	// Do what Qt needs to do
	QListViewItem::paintCell( p, cg, column, width, align );
	// And now draw the boxes around the cells
	// convert to physical column because potentially 
	// phys cols and logical cols could be different
	// e.g. when the user moved a column
	int pcol = listView()->header()->mapToActual(column);
	// Lower hor. line
	p->drawLine(0,height()-1,listView()->columnWidth(pcol)-1,height()-1);
	// vertical line
	p->drawLine(listView()->columnWidth(pcol)-1,0,listView()->columnWidth(pcol)-1,height());
}
*/

void row::paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int align )
{
	// Check if number alignment is allowed
	if (allowNumberAlign)
	{
		// Check if it is number and align it accordingly
		if (General::isNumber(text(column))) 
		{
			align=AlignRight|AlignVCenter;
		}
	}

    	// Do what Qt needs to do
	QListViewItem::paintCell( p, cg, column, width, align );
	// And now draw the boxes around the cells
	// Lower hor. line
	p->drawLine(0,height()-1,listView()->columnWidth(column)-1,height()-1);
	// vertical line
	p->drawLine(listView()->columnWidth(column)-1,0,listView()->columnWidth(column)-1,height());
}                                                                                                                         

void row::paintFocus ( QPainter * p, const QColorGroup & cg, const QRect & r )
{}

const QString row::getCellbyName (const QString colname) const
{
	QHeader *h = listView()->header();

	// search for the index of the colname specified
	for (int i=0; i<=h->count(); i++)
	{
		if (h->label(i).upper()==colname.upper())
		{
			return text(i);
		}
	}
	cerr << "No such column: " << colname << endl;
	return "";
}
