/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "scrollview.h" 
#include "general.h" 

#include <qpaintdevicemetrics.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qobjectlist.h>
// For qt3
#include <qpainter.h>

//ScrollView::ScrollView(QWidget * parent, const char * name) 
ScrollView::ScrollView(QWidget * parent, const QString& name) 
	:QScrollView(parent,name.local8Bit())
{
}

void ScrollView::contentsMousePressEvent ( QMouseEvent * e)
{
    if (e->button()==RightButton)
        emit rightButtonPressed(e->globalPos());
}
void ScrollView::showEvent(QShowEvent * e)
{
    emit shown();
}
 
void ScrollView::hideEvent(QHideEvent * e)
{
    emit hidden();
}                                                                               

void ScrollView:: printData(const QString& OutputFilename,
                            const QString& docName,
                            const QPrinter::Orientation& orient)
{
        const int Margin = 10;
        QPrinter *printer = new QPrinter();
        printer->setOutputFileName(OutputFilename);
        printer->setDocName(docName);
        printer->setOrientation(orient);

        if ( printer->setup(this) )
        {
                QPainter p;
                p.begin( printer );                     // paint on printer
                //int yPos        = 0;                    // y position for each line
                QFontMetrics fm = p.fontMetrics();
                QPaintDeviceMetrics metrics( printer ); // need width/height
                                                        // of printer surface

                QObjectList *list = viewport()->queryList("QLabel");
                QObjectListIt it( *list );          // iterate over the labels
                QObject * obj;
                QLabel* lbl;
                while ( (obj=it.current()) != 0 )
                { // for each found object...
                        ++it;
                        lbl = (QLabel*)obj;
                        if (lbl->frameStyle()!=QFrame::NoFrame) // so it's a text
                        {
                                p.drawRect( Margin+childX(lbl), Margin + childY(lbl),lbl->width(),
                                            fm.lineSpacing()+fm.underlinePos());
                                int align;
                                QString text=lbl->text();
                                //QRegExp r = QRegExp("^[0-9][0-9]*$",false,false);
                                //if (text.contains(r) > 0)
                                if (General::isNumber(text))
                                        align=ExpandTabs|AlignRight;
                                        //align=ExpandTabs|AlignRight|AlignVCenter;
                                else
                                        align=ExpandTabs;
                                p.drawText( Margin+childX(lbl), Margin + childY(lbl),lbl->width(),
                                            lbl->height(), align , text);
                        }
                        else // it's a label
                        {
                                p.drawText( Margin+childX(lbl), Margin + childY(lbl),lbl->width(),
                                            lbl->height(), ExpandTabs , lbl->text());
                        }
                }

                // print all multilineedits
                QObjectList *list_mle = viewport()->queryList("QMultiLineEdit");
                QObjectListIt it_mle( *list_mle );          // iterate over the labels
                QMultiLineEdit* mle;
                while ( (obj=it_mle.current()) != 0 )
                { // for each found object...
                        ++it_mle;
                        mle = (QMultiLineEdit*)obj;
                        p.drawRect( Margin+childX(mle), Margin + childY(mle),mle->width(),
                                    mle->height());
                        p.drawText( Margin+childX(mle), Margin + childY(mle),mle->width(),
                                    mle->height(), ExpandTabs , mle->text());

                }
                delete list;                        // delete the list, not the objects
		list=0;
                delete list_mle;                        // delete the list, not the objects
		list_mle=0;
                p.end();                                // send job to printer
        }
}

void ScrollView:: saveData()
{
        QString filename=QFileDialog::getSaveFileName("", "*.txt", this );
        if ( !filename.isNull() )
        {
                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream ts(&f);

                        QObjectList *list = viewport()->queryList("QFrame");
                        QObjectListIt it_label( *list );          // iterate over the labels
                        QObjectListIt it_text( *list );          // iterate over the text fields
                        QObject * obj;
                        QLabel* lbl;
                        QMultiLineEdit* mle;

                        // save the labels
                        while ( (obj=it_label.current()) != 0 )
                        { // for each found object...
                                ++it_label;
                                lbl = (QLabel*)obj;
                                if (lbl->frameStyle()==QFrame::NoFrame) // it's a label
                                {
                                        ts << lbl->text() << "\t";
                                }
                        }
                        ts << endl << endl;

                        // save the text
                        while ( (obj=it_text.current()) != 0 )
                        { // for each found object...
                                ++it_text;
                                if ((obj->className())==QString("QLabel"))
                                {
                                        lbl = (QLabel*)obj;
                                        if (lbl->frameStyle()!=QFrame::NoFrame) // so it's a text
                                        {
                                                if (lbl->text()==QString::null)
                                                        ts << "(null)" << "\t";
                                                else
                                                        ts << lbl->text()  << "\t";
                                        }
                                }
                                else if (obj->className()==QString("QMultiLineEdit"))
                                {
                                        mle = (QMultiLineEdit *)obj;
                                        if (lbl->text()==QString::null)
                                                ts << "(null)" << "\t";
                                        else
                                                ts << mle->text() << "\t";
                                }
                        }

                        delete list;                        // delete the list, not the objects
			list=0;
                }
        }
}
