/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include "sheet.h" 
#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <iostream.h>
#include <qpixmap.h>
#include <qmenubar.h>
#include "closesheet.xpm"
#include "print.xpm"
#include "mdi.h"

extern int gDefSheetWidth;
extern int gDefSheetHeight;

//Sheet::Sheet(QWidget * parent, const char * name)
Sheet::Sheet(QWidget * parent, const QString& name)
        : Window(gMDI->getWorkspace(), name),actionMenu(0), actionToolBar(0)
        //: Window(parent, name),actionToolBar(0)
{
	construct(name);
} 

Sheet::Sheet(const ObjParm& parm, QWidget * parent, const QString& name)
        : Window(parm, gMDI->getWorkspace(), name),actionMenu(0), actionToolBar(0)
        //: Window(parm, parent, name),actionToolBar(0)
{
	construct(name);
}

Sheet::~Sheet()
{
#ifdef DESTRACE
	cerr << "Sheet destructor" << endl;
#endif
	gMDI->unregisterSheet(MDIIndex);
	// Normally deleting the actionmenu causes the action menu item 
	// to be deleted as well. But it depends on whether the user has opened
	// the context menu (right button click) before the window is closed
	// If the user did this then Qt dissociates the menu from the menu bar
	// Thus the menu has no more parent. Then when it is deleted it does 
	// of course not delete the item anymore. This causes the problem that 
	// next time somebody accesses this menu item (that still thinks it 
	// has a menu underneath) you get a seg fault.
	//delete actionMenu;
	delete actionToolBar;
	actionToolBar=0;
	// This test here tells if the window currently being closed
	// is the active one or not
	// if it is the menu must be changed if not then
        // the menu has already been changed during focus switch
	// This here is in fact only really necessary when we close the 
	// last sheet because in all other cases the newly activated sheet
	// will change menu anyway
	if (gMDI->getActiveSheet()==this) 
	{
		// To solve the above problem we call removeActionItem() 
		// to remove the action menu item in any case
		gMDI->removeActionItem();
		gMDI->setActionToolBar(0);
		gMDI->setActionMenu(0);
	}
	delete actionMenu;
	actionMenu=0;
}

//void Sheet::construct(const char * name)
void Sheet::construct(const QString& name)
{
	MDIIndex=gMDI->registerSheet(this);
	OriginalSheetTitle=name;
        setCaption(OriginalSheetTitle);
	createActionMenu();
	createActionToolBar();
        actionToolBar->show();
	resize(gDefSheetWidth,gDefSheetHeight);
}

QPopupMenu* Sheet::getActionMenu() const
{
	return(actionMenu);
}

QToolBar* Sheet::getActionToolBar() const
{
	return(actionToolBar);
}

void Sheet::replaceMDIActionMenu()
{
	gMDI->replaceActionMenu(getActionMenu());
}

void Sheet::replaceMDIActionToolbar()
{
	gMDI->replaceActionToolbar(getActionToolBar());
}

void Sheet::ContextMenu(const QPoint & where)
{
        getActionMenu()->popup(where);
}

void Sheet::createActionMenu()
{
	// don't attach it to a parent
	// the deletion is managed by the class itself
    	actionMenu = new QPopupMenu();
    	//gMDI->setActionMenu( actionMenu );
        QPixmap closeIcon ;
        closeIcon = QPixmap( closesheet );
	getActionMenu()->insertItem(closeIcon, "Close sheet",this,SLOT(close()));
        getActionMenu()->insertSeparator();
        QPixmap printIcon ;
        printIcon = QPixmap( print );
        getActionMenu()->insertItem(printIcon, "Print...",this,SLOT(Print()));
}

const QString Sheet::getOriginalSheetTitle() const 
{
	return OriginalSheetTitle;
}
 
void Sheet::createActionToolBar()
{
        actionToolBar=new QToolBar("action Toolbar",gMDI,Left);
        QPixmap closeIcon ;
        closeIcon = QPixmap( closesheet );
        QToolButton * closeSheet = new QToolButton( closeIcon, "Close Sheet",0,
                                            this, SLOT(close()),
                                            actionToolBar, "close sheet" );
 
        actionToolBar->show();                                                  

        actionToolBar->addSeparator();

        QPixmap printIcon ;
        printIcon = QPixmap( print );
        QToolButton * print = new QToolButton( printIcon, "Print", 0,
                                            this, SLOT(Print()),
                                            actionToolBar, "Print" );
        print->show();

}
