/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include "row.h"
#include "dbtab.h"
#include "dbform.h"
#include "griddatadisplay.h"
#include "formdatadisplay.h"
#include "w_db_overview.h"
#include "dblist.h"
#include "objparm.h"
#include "qt_oraconnect.h"
#include "sqlcache.h"
#include "scrollview.h"
#include <qobjcoll.h>
#include <qtoolbutton.h>

w_db_overview::w_db_overview()
	: DBTab(0,"Database overview"),ftbs(0),fdf(0),frbs(0),frdl(0),
		      frdlg(0),fusers(0),fdbsize(0)
{
	//setCaption("Database overview");
	makeTabs();
        show();
	// from now on changing to the structure tab should fill it
	// This connect must be here. If you put it earlier, the fill code 
	// will be executed already during window creation making this much slower
// NO LONGER NECESSARY AS RETRIEVE IS REIMPLEMENTED
//	connect((TabDataDisplay*)getDD(), SIGNAL(selected(const QString &)), this, SLOT(fillStructureTab(const QString & )));
}

void w_db_overview::makeTabs()
{
	addFormTab("v_database","Database",1,190,130);
	makeStructureTab();
	addGridTab("sys_props","Properties");
}

void w_db_overview::makeStructureTab()
{
	const int startx=10;
	const int starty=50;
	const int lw=125;
	const int lh=20;
	const int fw=90;
	const int fh=20;
	const int hor_dist=10;
	const int ver_dist=20;
	const int col_dist=30;
	const int startx_col2=startx+lw+hor_dist+fw+col_dist;

	// Make it now so that it appears in the tab bar
	StructureTab=new ScrollView(this);
	StructureTab->resizeContents(530,289);
	StructureTab->setVScrollBarMode(QScrollView::Auto);
	StructureTab->setHScrollBarMode(QScrollView::Auto);
	StructureTab->enableClipper(TRUE);

	StructureTab->viewport()->setBackgroundMode(PaletteBase);

    	connect(StructureTab, SIGNAL(shown()), this, SLOT(StructureTabShown()));
    	connect(StructureTab, SIGNAL(hidden()), this, SLOT(StructureTabHidden()));
    	connect(StructureTab, SIGNAL(rightButtonPressed(const QPoint &)), this, SLOT(ContextMenu(const QPoint &)));       

        // Tablespaces
        QLabel * l = new QLabel("Tablespaces:",StructureTab->viewport());
	l->resize(lw,lh);
	l->setBackgroundMode(PaletteBase);
        StructureTab->addChild(l,startx,starty+0*(lh+ver_dist));

        ftbs = new QLabel("",StructureTab->viewport());
        ftbs->resize(fw,fh);
	ftbs->setBackgroundMode(PaletteBase);
        StructureTab->addChild(ftbs,startx+lw+hor_dist,starty+0*(fh+ver_dist));
        ftbs->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        ftbs->setFrameStyle(QFrame::Box|QFrame::Plain);

        // Datafiles
        l = new QLabel("Datafiles:",StructureTab->viewport());
	l->resize(lw,lh);
	l->setBackgroundMode(PaletteBase);
        StructureTab->addChild(l,startx,starty+1*(lh+ver_dist));

        fdf = new QLabel("",StructureTab->viewport());
        fdf->resize(fw,fh);
	fdf->setBackgroundMode(PaletteBase);
        StructureTab->addChild(fdf,startx+lw+hor_dist,starty+1*(lh+ver_dist));
        fdf->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        fdf->setFrameStyle(QFrame::Box|QFrame::Plain);

        // Rollback segments
        l = new QLabel("Rollback segments:",StructureTab->viewport());
        l->resize(lw,lh);
	l->setBackgroundMode(PaletteBase);
        StructureTab->addChild(l,startx,starty+2*(lh+ver_dist));

        frbs = new QLabel("",StructureTab->viewport());
        frbs->resize(fw,fh);
	frbs->setBackgroundMode(PaletteBase);
        StructureTab->addChild(frbs,startx+lw+hor_dist,starty+2*(fh+ver_dist));
        frbs->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        frbs->setFrameStyle(QFrame::Box|QFrame::Plain);

        // Redo log files
        l = new QLabel("Redo log files:",StructureTab->viewport());
        l->setGeometry(startx,starty+3*(lh+ver_dist),lw,lh);
        l->resize(lw,lh);
	l->setBackgroundMode(PaletteBase);
        StructureTab->addChild(l,startx,starty+3*(lh+ver_dist));

        frdl = new QLabel("",StructureTab->viewport());
        frdl->resize(fw,fh);
	frdl->setBackgroundMode(PaletteBase);
        StructureTab->addChild(frdl,startx+lw+hor_dist,starty+3*(fh+ver_dist));
        frdl->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        frdl->setFrameStyle(QFrame::Box|QFrame::Plain);

        // Redo log groups
        l = new QLabel("Groups:",StructureTab->viewport());
        l->resize(lw,lh);
	l->setBackgroundMode(PaletteBase);
        StructureTab->addChild(l,startx_col2,starty+3*(lh+ver_dist));

        frdlg = new QLabel("",StructureTab->viewport());
        frdlg->resize(fw,fh);
	frdlg->setBackgroundMode(PaletteBase);
        StructureTab->addChild(frdlg,startx_col2+lw+hor_dist,starty+3*(fh+ver_dist));
        frdlg->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        frdlg->setFrameStyle(QFrame::Box|QFrame::Plain);

        // Users
        l = new QLabel("Users:",StructureTab->viewport());
        l->setGeometry(startx,starty+4*(lh+ver_dist),lw,lh);
        l->resize(lw,lh);
	l->setBackgroundMode(PaletteBase);
        StructureTab->addChild(l,startx,starty+4*(lh+ver_dist));

        fusers = new QLabel("",StructureTab->viewport());
        fusers->resize(fw,fh);
	fusers->setBackgroundMode(PaletteBase);
        StructureTab->addChild(fusers,startx+lw+hor_dist,starty+4*(fh+ver_dist));
        fusers->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        fusers->setFrameStyle(QFrame::Box|QFrame::Plain);

        // DbSize
        l = new QLabel("Database size [MB]:",StructureTab->viewport());
        l->setGeometry(startx,starty+5*(lh+ver_dist),lw,lh);
        l->resize(lw,lh);
	l->setBackgroundMode(PaletteBase);
        StructureTab->addChild(l,startx,starty+5*(lh+ver_dist));

        fdbsize = new QLabel("",StructureTab->viewport());
        fdbsize->resize(fw,fh);
	fdbsize->setBackgroundMode(PaletteBase);
        StructureTab->addChild(fdbsize,startx+lw+hor_dist,starty+5*(fh+ver_dist));
        fdbsize->setAlignment(QFrame::AlignRight|QFrame::AlignVCenter);
        fdbsize->setFrameStyle(QFrame::Box|QFrame::Plain);

	addTab(StructureTab,"Structure");
}

void w_db_overview::fillStructureTab(const QString & tabname)
{
	if (tabname=="Structure")
	{
           if (!gSQLCache)
           {
                QMessageBox::critical( 0, "SQL cache error", QString("The SQL cache is not set up"),QMessageBox::Ok | QMessageBox::Default,0,0);
                return;
           }
           if (!gConnObj)
           {
                QMessageBox::critical( 0, "Database error", "You are not connected to a database!",QMessageBox::Ok | QMessageBox::Default,0,0);
                return;
           }
	   qApp->setOverrideCursor(waitCursor);
	   try
	   {
		// Execute sql from the cache file
		gConnObj->open(gSQLCache->getSQL("tbs_no"));
		gConnObj->fetch();
        	ftbs->setText(gConnObj->getCol(0));
		gConnObj->close();

		// Execute sql from the cache file
		gConnObj->open(gSQLCache->getSQL("rbs_no"));
		gConnObj->fetch();
        	frbs->setText(gConnObj->getCol(0));
		gConnObj->close();

		// Execute sql from the cache file
		gConnObj->open(gSQLCache->getSQL("file_no_dbsize"));
		gConnObj->fetch();
        	fdf->setText(gConnObj->getCol(0));
        	fdbsize->setText(gConnObj->getCol(1));
		gConnObj->close();

		// Execute sql from the cache file
		gConnObj->open(gSQLCache->getSQL("redo_no"));
		gConnObj->fetch();
        	frdl->setText(gConnObj->getCol(0));
        	frdlg->setText(gConnObj->getCol(1));
		gConnObj->close();

		// Execute sql from the cache file
		gConnObj->open(gSQLCache->getSQL("user_no"));
		gConnObj->fetch();
        	fusers->setText(gConnObj->getCol(0));
		gConnObj->close();
	   }
	   catch (oerr & e)
	   {
	   	qApp->restoreOverrideCursor();
		QMessageBox::critical( 0, "Database error", ((qt_oerr&)e).getMsg(),QMessageBox::Ok | QMessageBox::Default,0,0);
		return;
	   }
	   qApp->restoreOverrideCursor();
	}
}

void w_db_overview::StructureTabShown()
{
    QPopupMenu * action_menu = getActionMenu();
    action_menu->setItemEnabled(action_menu->idAt(6),false);
    action_menu->setItemEnabled(action_menu->idAt(7),false);
    action_menu->setItemEnabled(action_menu->idAt(8),false);
    action_menu->setItemEnabled(action_menu->idAt(9),false);
    action_menu->setItemEnabled(action_menu->idAt(10),false);
 
    QObjectList *list = ((QObject *)getActionToolBar())->queryList("QToolButton");
    QObjectListIt it( *list );          // iterate over the buttons
    QObject * obj;
    while ( (obj=it.current()) != 0 )
    { // for each found object...
        //if (((QString)((QToolButton *)obj)->name())=="Save as"
        if (((QString)((QToolButton *)obj)->name())=="Sort"
        ||  ((QString)((QToolButton *)obj)->name())=="Filter"
        ||  ((QString)((QToolButton *)obj)->name())=="SQL")
        {
            ((QToolButton *)obj)->setEnabled(false);
        }
        ++it;
    }
}

void w_db_overview::StructureTabHidden()
{
    QPopupMenu * action_menu = getActionMenu();
    action_menu->setItemEnabled(action_menu->idAt(6),true);
    action_menu->setItemEnabled(action_menu->idAt(7),true);
    action_menu->setItemEnabled(action_menu->idAt(8),true);
    action_menu->setItemEnabled(action_menu->idAt(9),true);
    action_menu->setItemEnabled(action_menu->idAt(10),true);
 
    QObjectList *list = ((QObject *)getActionToolBar())->queryList("QToolButton");
    QObjectListIt it( *list );          // iterate over the buttons
    QObject * obj;
    while ( (obj=it.current()) != 0 )
    { // for each found object...
        //if (((QString)((QToolButton *)obj)->name())=="Save as"
        if (((QString)((QToolButton *)obj)->name())=="Sort"
        ||  ((QString)((QToolButton *)obj)->name())=="Filter"
        ||  ((QString)((QToolButton *)obj)->name())=="SQL")
        {
            ((QToolButton *)obj)->setEnabled(true);
        }
        ++it;
    }
}

void w_db_overview::retrieve()
{
    QString tabname=((TabDataDisplay *)getDD())->getCurrentTab();
    if (tabname!="Structure")
    {
        DBSheet::retrieve();
    }
    else
    {
        fillStructureTab(tabname);
    }
}

void w_db_overview::Print()
{
    QString tabname=((TabDataDisplay *)getDD())->getCurrentTab();
    if (tabname!="Structure")
    {
        DBSheet::Print();
    }
    else
    {
	StructureTab->printData();
    }
}

void w_db_overview::SaveAs()
{
    QString tabname=((TabDataDisplay *)getDD())->getCurrentTab();
    if (tabname!="Structure")
    {
        DBSheet::SaveAs();
    }
    else
    {
        StructureTab->saveData();
    }
}
