/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include "w_dba_tab_partitions.h"
#include "w_dba_tab_partitions_info.h"

#include <stdlib.h>
#include <qmessagebox.h>
#include "scriptdlg.h"
#include "gensql.h"
#include "editor.h"
#include "config.h"
#include <qfile.h>
#include <qtextstream.h>

w_dba_tab_partitions::w_dba_tab_partitions()
	: DBList("dba_tab_partitions",0,"DBA_TAB_PARTITIONS")
{
//	addGenerateSQLToActionMenu();
	show();
}

w_dba_tab_partitions::w_dba_tab_partitions(const ObjParm& parm)
	: DBList(parm,"dba_tab_partitions_per_user",0,"DBA_TAB_PARTITIONS for user "+parm.getParm(0))
{
//	addGenerateSQLToActionMenu();
	show();
}

QPopupMenu * w_dba_tab_partitions::addGenerateSQLToActionMenu()
{
        QPopupMenu * m= DBList::addGenerateSQLToActionMenu();
        m->insertItem("Drop...",this,SLOT(generate_tab_partition_drop()));
        return m;
}

void w_dba_tab_partitions::callInfo(row * clickedRow)
{
	ObjParm parm;
        if (getWinParm())
        { // We are in the per_user case
		parm.addParm(getWinParm()->getParm(0)); // username
        }
        else
        { // We are in the all objects case
                parm.addParm(clickedRow->getCellbyName("table_owner"));
        }
	parm.addParm(clickedRow->getCellbyName("table_name"));
	parm.addParm(clickedRow->getCellbyName("partition_name"));
	w_dba_tab_partitions_info *w = new w_dba_tab_partitions_info(parm);
}

void w_dba_tab_partitions::generate_tab_partition_drop()
{
        generate_sql("tab_partition_drop.sql","tab_partition_drop");
}

void w_dba_tab_partitions::generate_sql(const QString& fn, const QString& what)
{
        QString owner,filename,object_name;
        bool    allobj, show_script, echo_on, spool;

        gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");

        ScriptDlg *sd = new ScriptDlg(path+"/"+fn,this);
        if (sd->exec())
        {
		qApp->setOverrideCursor(waitCursor);
                filename=sd->getFilename();
                allobj=sd->getAllObj();
                show_script=sd->getShow();
                spool=sd->getSpool();
                echo_on=sd->getEchoOn();
                delete sd;
		sd=0;

                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream t(&f);
                        if (spool) t << "spool " << filename << ".log" << endl;
                        if (echo_on) t << "set echo on" << endl;
                        if (!allobj) // generate script for selected object
                        {
                                object_name=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("name");
                                if (getWinParm())
                                { // We are in the per_user case
                                        owner=getWinParm()->getParm(0);
                                }
                                else
                                { // We are in the all objects case
                                        owner=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("owner");
                                }
                                int retval=0;
                                //if (what=="tab_partition_drop")
                                 //       retval=GenSQL::PartitionDrop(t,owner, object_name);
                                if (retval==-1)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-2)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-3)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                        }
                        else // generate script for all objects
                        {
                                QListViewItemIterator it((GridDataDisplay *)getDD());
                                // loop over all rows
                                for (;it.current(); ++it)
                                {
                                        object_name=((row *)it.current())->getCellbyName("name");
                                        if (getWinParm())
                                        { // We are in the per_user case
                                                owner=getWinParm()->getParm(0);
                                        }
                                        else
                                        { // We are in the all objects case
                                                owner=((row *)it.current())->getCellbyName("owner");
                                        }
                                        int retval=0;
                                        if (what=="tab_partition_drop")
                                                retval=GenSQL::OutlineDrop(t,owner,object_name);
                                        if (retval==-1)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-2)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-3)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                }
                        }
                        if (echo_on) t << "set echo off" << endl;
                        if (spool) t << "spool off" << endl;
                        f.close();
                }

                if (show_script)
                {
                        Editor * ed = new Editor(filename);
                        ed->show();
                }
		qApp->restoreOverrideCursor();
        }
        else
        {
                delete sd;
		sd=0;
        }
}

