/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include <stdlib.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qtextstream.h>
#include "w_dba_tables.h"
#include "w_dba_tables_info.h"
#include "gensql.h"
#include "editor.h"
#include "scriptdlg.h"
#include "config.h"

w_dba_tables::w_dba_tables()
	: DBList("dba_tables",0,"DBA_TABLES")
{
	addGenerateSQLToActionMenu();
	show();
}

w_dba_tables::w_dba_tables(const ObjParm& parm)
	: DBList(parm,"dba_tables_per_user",0,"DBA_TABLES for user "+parm.getParm(0))
{
        //setCaption("DBA_TABLES for user "+parm.getParm(0));
	addGenerateSQLToActionMenu();
	show();
}

QPopupMenu * w_dba_tables::addGenerateSQLToActionMenu()
{
        QPopupMenu * m= DBList::addGenerateSQLToActionMenu();
        m->insertItem("Create...",this,SLOT(generate_table_create()));
        m->insertItem("Create with constraints...",this,SLOT(generate_table_create_with_constraints()));
        m->insertItem("Drop...",this,SLOT(generate_table_drop()));
        m->insertItem("Truncate...",this,SLOT(generate_table_truncate()));
        m->insertItem("Deallocate...",this,SLOT(generate_table_deallocate()));
        return m;
}

void w_dba_tables::callInfo(row * clickedRow)
{
	ObjParm parm;
	if (getWinParm())
	{ // We are in the per_user case
		parm.addParm(getWinParm()->getParm(0)); // username
	}
	else
	{ // We are in the all objects case
		parm.addParm(clickedRow->getCellbyName("owner"));
	}
	parm.addParm(clickedRow->getCellbyName("table_name"));
	w_dba_tables_info *w = new w_dba_tables_info(parm);
}

void w_dba_tables::generate_table_create()
{
        generate_sql("table_create.sql","table_create");
}

void w_dba_tables::generate_table_create_with_constraints()
{
        generate_sql("table_create_with_constraints.sql","table_create_with_constraints");
}

void w_dba_tables::generate_table_drop()
{
        generate_sql("table_drop.sql","table_drop");
}

void w_dba_tables::generate_table_truncate()
{
        generate_sql("table_truncate.sql","table_truncate");
}

void w_dba_tables::generate_table_deallocate()
{
        generate_sql("table_deallocate.sql","table_deallocate");
}
void w_dba_tables::generate_sql(const QString& fn, const QString& what)
{
        QString owner,filename,object_name;
        bool    allobj, show_script, echo_on, spool;

        gConfigFile->setGroup("PATH");
        QString path = gConfigFile->readEntry("sql_generation",".");

        ScriptDlg *sd = new ScriptDlg(path+"/"+fn,this);
        if (sd->exec())
        {
		qApp->setOverrideCursor(waitCursor);
                filename=sd->getFilename();
                allobj=sd->getAllObj();
                show_script=sd->getShow();
                spool=sd->getSpool();
                echo_on=sd->getEchoOn();
                delete sd;
		sd=0;

                QFile f(filename);
                if (f.open(IO_WriteOnly))
                {
                        QTextStream t(&f);
                        if (spool) t << "spool " << filename << ".log" << endl;
                        if (echo_on) t << "set echo on" << endl;
                        if (!allobj) // generate script for selected object
                        {
                                object_name=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("table_name");
                                if (getWinParm())
                                { // We are in the per_user case
                                        owner=getWinParm()->getParm(0);
                                }
                                else
                                { // We are in the all objects case
                                        owner=((row *)(((GridDataDisplay *)getDD())->currentItem()))->getCellbyName("owner");
                                }
                                int retval=0;
                                if (what=="table_create")
                                	retval=GenSQL::TableCreate(t,owner,object_name);
                                else if (what=="table_drop")
                                	retval=GenSQL::TableDrop(t,owner,object_name);
                                else if (what=="table_truncate")
                                	retval=GenSQL::TableTruncate(t,owner,object_name);
                                else if (what=="table_deallocate")
                                	retval=GenSQL::TableDeallocate(t,owner,object_name);
                                else if (what=="table_create_with_constraints")
                                	retval=GenSQL::TableCreateWithConstraints(t,owner,object_name);
                                if (retval==-1)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-2)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                                else if (retval==-3)
                                {
                                        QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                        f.close();
                                        return;
                                }
                        }
                        else // generate script for all objects
                        {
                                QListViewItemIterator it((GridDataDisplay *)getDD());
                                // loop over all rows
                                for (;it.current(); ++it)
                                {
                                        object_name=((row *)it.current())->getCellbyName("table_name");
                                        if (getWinParm())
                                        { // We are in the per_user case
                                                owner=getWinParm()->getParm(0);
                                        }
                                        else
                                        { // We are in the all objects case
                                                owner=((row *)it.current())->getCellbyName("owner");
                                        }
                                        int retval=0;
                                        if (what=="table_create")
                                                retval=GenSQL::TableCreate(t,owner,object_name);
                                        else if (what=="table_drop")
                                                retval=GenSQL::TableDrop(t,owner,object_name);
                                        else if (what=="table_truncate")
                                                retval=GenSQL::TableTruncate(t,owner,object_name);
                                        else if (what=="table_deallocate")
                                                retval=GenSQL::TableDeallocate(t,owner,object_name);
                                        else if (what=="table_create_with_constraints")
                                                retval=GenSQL::TableCreateWithConstraints(t,owner,object_name);
                                        if (retval==-1)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "File write error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-2)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "Database error. Processing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                        else if (retval==-3)
                                        {
                                                QMessageBox::critical( 0, "SQL generation error", "SQL cache error. SQL could not be found!\nProcessing stopped!",QMessageBox::Ok | QMessageBox::Default,0,0);
                                                f.close();
                                                return;
                                        }
                                }
                        }
                        if (echo_on) t << "set echo off" << endl;
                        if (spool) t << "spool off" << endl;
                        f.close();
                }

                if (show_script)
                {
                        Editor * ed = new Editor(filename);
                        ed->show();
                }
		qApp->restoreOverrideCursor();
        }
        else
        {
                delete sd;
		sd=0;
        }
}

