/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2000-2003  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/
#include <iostream.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qstring.h>
#include "dbobjects.h"
#include "row.h"
#include "dbtab.h"
#include "dbform.h"
#include "griddatadisplay.h"
#include "formdatadisplay.h"
#include "w_dba_users_info.h"
#include "mdi.h"
#include "dblist.h"
#include "objparm.h"
#include "sqlcache.h"
#include "qt_oraconnect.h"
#include <qmessagebox.h>
#include <qobjcoll.h>
#include <qtoolbutton.h>

w_dba_users_info::w_dba_users_info(const ObjParm& parm)
	: DBTab(parm,0,"User information for "+parm.getParm(0)), 
	  previousTab(QString::null)
{
	//setCaption("User information for "+parm.getParm(0));
	makeTabs();
        show();
}

void w_dba_users_info::makeTabs()
{
	makeObjectsTab();
	addGridTab("dba_sys_privs_per_user","Sys Privs");
        addGridTab("dba_role_privs_per_user","Role Privs");
        addGridTab("dba_tab_privs_per_user","Tab Privs");
        addGridTab("dba_col_privs_per_user","Col Privs");
//	makeObjectsTab();
        addGridTab("dba_profiles_per_user","Profiles");
        addGridTab("dba_ts_quotas_per_user","TS Quotas");
        addGridTab("dba_errors_per_user","Errors");
	connect((TabDataDisplay*)getDD(), SIGNAL(selected(const QString &)), this, SLOT (ManageObjTab(const QString &)));
}


void w_dba_users_info::makeObjectsTab()
{
	ObjTab = new DBObjects(getWinParm()->getParm(0),this);
	//connect(ObjTab, SIGNAL(shown()), this, SLOT(ObjectTabShown()));
        //connect(ObjTab, SIGNAL(hidden()), this, SLOT(ObjectTabHidden()));
        connect(ObjTab, SIGNAL(rightButtonPressed(const QPoint &)), this, SLOT(ContextMenu(const QPoint &)));
	//ObjTab->resize(500,300);
	addTab(ObjTab,"Objects");
}

void w_dba_users_info::ObjectTabShown()
{
	addCalcMenu();
	QPopupMenu * action_menu = gMDI->getActionMenu();
	action_menu->setItemEnabled(action_menu->idAt(5),false);
	action_menu->setItemEnabled(action_menu->idAt(6),false);
	action_menu->setItemEnabled(action_menu->idAt(7),false);
	action_menu->setItemEnabled(action_menu->idAt(8),false);
	action_menu->setItemEnabled(action_menu->idAt(9),false);
	action_menu->setItemEnabled(action_menu->idAt(10),false);

    QObjectList *list = ((QObject *)getActionToolBar())->queryList("QToolButton");
    QObjectListIt it( *list );          // iterate over the buttons
    QObject * obj;
    while ( (obj=it.current()) != 0 )
    { // for each found object...
        //if (((QString)((QToolButton *)obj)->name())=="Save as"
        if (((QString)((QToolButton *)obj)->name())=="Refresh"
        ||  ((QString)((QToolButton *)obj)->name())=="Sort"
        ||  ((QString)((QToolButton *)obj)->name())=="Filter"
        ||  ((QString)((QToolButton *)obj)->name())=="SQL")
        {
            ((QToolButton *)obj)->setEnabled(false);
        }
        ++it;
    }                                                                                                             
}

void w_dba_users_info::ObjectTabHidden()
{
    removeCalcMenu();
	QPopupMenu * action_menu = gMDI->getActionMenu();
	action_menu->setItemEnabled(action_menu->idAt(5),true);
	action_menu->setItemEnabled(action_menu->idAt(6),true);
	action_menu->setItemEnabled(action_menu->idAt(7),true);
	action_menu->setItemEnabled(action_menu->idAt(8),true);
	action_menu->setItemEnabled(action_menu->idAt(9),true);
	action_menu->setItemEnabled(action_menu->idAt(10),true);

    QObjectList *list = ((QObject *)getActionToolBar())->queryList("QToolButton");
    QObjectListIt it( *list );          // iterate over the buttons
    QObject * obj;
    while ( (obj=it.current()) != 0 )
    { // for each found object...
        //if (((QString)((QToolButton *)obj)->name())=="Save as"
        if (((QString)((QToolButton *)obj)->name())=="Refresh"
        ||  ((QString)((QToolButton *)obj)->name())=="Sort"
        ||  ((QString)((QToolButton *)obj)->name())=="Filter"
        ||  ((QString)((QToolButton *)obj)->name())=="SQL")
        {
            ((QToolButton *)obj)->setEnabled(true);
        }
        ++it;
    }                                                                                                        
}

void w_dba_users_info::addCalcMenu()
{
        QPopupMenu * action_menu = gMDI->getActionMenu();
        calc_separator_id=action_menu->insertSeparator();

        calc_menu = new QPopupMenu(this);
        calc_menu_id=action_menu->insertItem("Calculate",calc_menu);

        calc_menu->insertItem("All",ObjTab,SLOT(calcAll()));
        calc_menu->insertSeparator();
        calc_menu->insertItem("All sizes",ObjTab,SLOT(calcAllSizeOfObjects()));
        calc_menu->insertItem("All numbers",ObjTab,SLOT(calcAllNoOfObjects()));
        calc_menu->insertSeparator();
        calc_menu->insertItem("Current size and number",ObjTab,SLOT(calcCurSizeAndNumber()));
        calc_menu->insertItem("Current number",ObjTab,SLOT(calcCurNoOfObjects()));
        calc_menu->insertItem("Current size",ObjTab,SLOT(calcCurSizeOfObjects()));
}

void w_dba_users_info::removeCalcMenu()
{
	QPopupMenu * action_menu = gMDI->getActionMenu();
	action_menu->removeItem(calc_menu_id);
	delete calc_menu;
	calc_menu=0;
	action_menu->removeItem(calc_separator_id);
}

void w_dba_users_info::Print()
{
	if (((TabDataDisplay*)getDD())->getCurrentTab()=="Objects")
		ObjTab->printData();
	else
		DBSheet::Print();
}

void w_dba_users_info::SaveAs()
{
	if (((TabDataDisplay*)getDD())->getCurrentTab()=="Objects")
		ObjTab->saveData();
	else
		DBSheet::SaveAs();
}

void w_dba_users_info::ManageObjTab(const QString& currentTab)
{
	if (currentTab!=previousTab)
	{
		if (currentTab=="Objects") ObjectTabShown();
		else if (previousTab=="Objects") ObjectTabHidden();

		previousTab=currentTab;
	}
}
