/****************************************************************************
 *
 * DBAComp(anion) - An administration tool for Oracle databases
 * Copyright (C) 2001  Torsten FASSIAN (torsten@internet.lu) 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *	As a special exception, you have permission to link this program
 *	with the Qt library and distribute executables, as long as you
 *	follow the requirements of the GNU GPL in regard to all of the
 *	software in the executable aside from Qt.
 *
 ****************************************************************************/

#ifndef ORACONNECT_H
#define ORACONNECT_H

struct SQLDA;

////////////////////////////
class oerr
{
public:
	oerr(const char* msg);
	oerr(const oerr& err);

	virtual ~oerr();

	const char* getMsg() const;
	virtual oerr& operator= (const oerr& err);

private:
	char* oramsg;
};

////////////////////////////
class OraParm
{

public:
        OraParm(const int num);
        OraParm(const char* parm=0);
        OraParm(const OraParm& parm);
	virtual ~OraParm();

        void addParm(const char* parm);
        virtual int ParmCount() const;
        const char* getParm(int index) const;
        OraParm& operator= (const char* parm);
        OraParm& operator= (const OraParm& parm);

private:
	typedef char * parameter;
        parameter* ParmList;
	int maxParm;
	int nextParm;
};

///////////////////////////
class oraconnect
{
public:
	// Create oraconnect instance and connect to a database
	oraconnect(const char *name,     // logical name of the connection
		   const char *uname=0,  // username
                   const char *pwd=0,    // password
                   const char *db=0);    // tnsalias
	~oraconnect(); 

	void open(const char * query); // Open Cursor without bind parms
	void open(const char * query, const OraParm& parm); // Open Cursor with
	int getCols(); // return the number of cols in the query
	char* getColName(int col); // return the name of cols in the query
	char* getCol(int col); // return the value of a column
	int fetch(); 
	long count(); // return number of rows fetched so far
	void close();             // Close Cursor


	typedef char* bindvar;
	typedef bindvar* bindvarlist;

protected:
	int alloc_bind_desc();
	int alloc_select_desc();
	void setSQL(const char* text);
	void prepare();
	void declareCursor();
	void descBind();
	void setBindVar(int pos, const char * bv);
	void descSelect();
	void reset_Dt_Len();

	void sql_error(); // Error Handler


private:
	SQLDA *bind_dp;
	SQLDA *select_dp;
	char* sql; // contains the SQL currently used
	int cols; // no of select list items
	int bind; // no of bind vars
	char *connection_name; // logical name of the current connection

	bool cursor_open;

	char * errmsg;  // holds the last err mesg. It is needed to convert
			// oracle message that is not terminated to a
			// terminated message

	// This is needed to convert select_dp->S[i] to a null terminated string
	// In the 8.0.5 it is not null terminated and in 8.1.5 it seems to be
	// In 8.1.6 it's again not terminated. According to the documentation
	// it should not be terminated. Thus I chose to add a terminator to it
	// in any case
	// ColNameWithTerm is the value of a column with a terminator
	char * ColNameWithTerm;
};
#endif
